/* Copyright (c) 2000-2001 ActiveState Corporation.
   See the file LICENSE.txt for licensing information. */

/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * editor-debugger.js
 * Set up the buffer- and window-manager components.
 */

//---- globals
var DBG = {
    terminate_observer: null,
    STATE_STOPPED: Components.interfaces.koIDebugger.DBG_STATE_STOPPED,
    STATE_STARTING: Components.interfaces.koIDebugger.DBG_STATE_STARTING,
    STATE_BREAK: Components.interfaces.koIDebugger.DBG_STATE_BREAK,
    STATE_RUNNING: Components.interfaces.koIDebugger.DBG_STATE_RUNNING,
    STATE_STOPPING: Components.interfaces.koIDebugger.DBG_STATE_STOPPING,
    
    INVOKE_RUN: Components.interfaces.koIInvocation.INVOKE_RUN,
    INVOKE_DEBUG: Components.interfaces.koIInvocation.INVOKE_DEBUG,
    INVOKE_BUILD: Components.interfaces.koIInvocation.INVOKE_BUILD,
    
    log: null,
    komodoQuitting: false,
    
    // Debugger Session Data
    session: null
};

var _debugger_stateNames = [
    // this use used for log file output only, and matches
    // the enumeration of these constants found in koIDebugger
    'DBG_STATE_STOPPED',
    'DBG_STATE_BREAK',
    'DBG_STATE_RUNNING',
    'DBG_STATE_STARTING',
    'DBG_STATE_STOPPING'
];

var Editor_Markers = {
    // markerOrdering: the lower the number, the higher the marker's z-order
    clb: 5, // current line background color
    bm: 4, // bookmark
    cl: 3, // current line
    wb: 2, // working (ie, enabled) breakpoint
    fb: 1 // false breakpoint
}

function DBG_CurrentMarker() {}
DBG_CurrentMarker.prototype.constructor = DBG_CurrentMarker;
DBG_CurrentMarker.prototype = {
    file: null,
    line: null,
    ifile: null,
    iline: null
}

function DBG_Session() {
    this.currentmarker = new DBG_CurrentMarker();
}
DBG_Session.prototype.constructor = DBG_Session;
DBG_Session.prototype = {
    ui_state: Components.interfaces.koIDebugger.DBG_STATE_STOPPED,
    cookie: null,
    start_in_progress: false,
    invocation: null,
    lastCommand: null,
    filename: "",
    input_filename: "",
    currentmarker: null,
    listener_manager: null,
    _dbg: null,
    _view: null,
    _terminal: null,
    
    // state information to use at debugger end
    viewAtDebuggerStart: null,
    lineAtDebuggerStart: -1,
    currentColumnAtDebuggerStart: -1,
    anchorAtDebuggerStart: -1,

    initTerminal: function() {
        uilayout_ensureOutputPaneShown(window);
        if (this.terminal.active)
            this.terminal.endSession();
        this.view.init();
        this.view.initWithTerminal(this.terminal);
        this.terminal.clear();
        this.terminal.startSession();
    },
    get view() {
        if (!this._view)
            this._view = document.getElementById("redirect-scin");
        return this._view;
    },
    get terminal() {
        if (!this._terminal)
            this._terminal =
                Components.classes['@activestate.com/koTerminalHandler;1']
                    .createInstance(Components.interfaces.koITerminalHandler);
        return this._terminal;
    },
    // Get the koIDebugger interface, creating and initializing if necessary.
    get dbg() {
        if (this._dbg == null) {
            //window.style.cursor = "wait"
            this._dbg = Components.classes["@activestate.com/debugger;1"].
                            getService(Components.interfaces.koIDebugger);
            this.cookie = this._dbg.addListener(this);
        }
        return this._dbg;
    },
    reset: function() {
        if (this._dbg) {
            if (this.cookie != null) {
                this._dbg.removeListener(this.cookie);
                this.cookie = null;
            }
            this._dbg = null;
            this.invocation = null;
        }
    },
    terminate: function() {
      if (this._dbg && this._dbg.state != DBG.STATE_STOPPED)
          this._dbg.stop();
      else
          debugger_change_ui_state(DBG.STATE_STOPPED);
      if (this.invocation) {
        try {
          this.invocation.terminate(99,1);
        } catch(e) {
          ; // already dead
        }
      }
    },
    onStateChange: function(state, state_extra) {
        // *sigh* - threads are too fast! The debugger may send a number of 
        // state requests before the first has fired.  Thus, the state we are 
        // notified of may not be the state we actually are!
        // So - ignore the state sent, and just query the current state.
    
        //XXX if Komodo is shutting down, we don't care what the 
        // debugger state is, and trying to exec debugger_change_ui_state()
        // may cause trouble, since some objects may already be released
        try {
            if (!DBG.komodoQuitting) {
                debugger_change_ui_state(this.dbg.state);
            }
        } catch(e) {
            DBG.log.exception(e);
        }
    },
    canStep: function() {
        try {
            if (!this.dbg)
                return true;
            if (this.ui_state == DBG.STATE_BREAK)
                return true;
            if (!gViewMgr.currentView ||
                gViewMgr.currentView.getAttribute('type') != 'editor' ||
                !gViewMgr.currentView.document ||
                !gViewMgr.currentView.document.file ||
                !gViewMgr.currentView.document.languageObj.debuggable)
                return false;
            return (this.ui_state == DBG.STATE_STOPPED);
        } catch(e) {
            DBG.log.error(e);
        }
        return false;
    },
    canRun: function() {
        try {
            // debuggable only means that the invocation is loadable
            // but it will at least prevent a run on anything that
            // does not have an invocation or debugger support in some way
            if (!gViewMgr.currentView ||
                gViewMgr.currentView.getAttribute('type') != 'editor' ||
                !gViewMgr.currentView.document ||
                !gViewMgr.currentView.document.file ||
                !gViewMgr.currentView.document.languageObj.debuggable)
                return false;
            return (this.ui_state == DBG.STATE_STOPPED);
        } catch(e) {
            DBG.log.error(e);
        }
        return false;
    },
    QueryInterface: function (iid) {
        if (!iid.equals(Components.interfaces.koIDebuggerEvents) &&
            !iid.equals(Components.interfaces.nsISupports)) {
            throw Components.results.NS_ERROR_NO_INTERFACE;
        }
        return this;
    }
};

//---- editor-debugger.js's observer
// This observer handles any notification pertinent to the UI side of
// debugging stuff in Komodo (i.e. stuff that editor-debugger.js deals with).
//
function _DebuggerObserver() {}
_DebuggerObserver.prototype.observe = function(subject, topic, data)
{
    switch (topic) {
    case 'invocation_terminated':
        // XXX "data" is null for now.
        //var exitCode = data;  // XXX can "data" be an int here???
        DBG.session.terminate();
        break;
    }
}



//---- module routines

function _debugger_is_local_host(host_name) {
    return host_name == "localhost" || host_name == "127.0.0.1";
}


function debugger_onLoad()
{
    KomodoRegisterOnUnloadHandler(debugger_onUnLoad);
    var _gObserverSvc = Components.classes["@mozilla.org/observer-service;1"].
                   getService(Components.interfaces.nsIObserverService);
    DBG.terminate_observer = new _DebuggerObserver();
    _gObserverSvc.addObserver(DBG.terminate_observer, "invocation_terminated",false);
    DBG.log = getLoggingMgr().getLogger('debugger');
    DBG.listener_manager = new listener_ui_manager(window);
    DBG.session = new DBG_Session();
    DBG.controller = new DebuggerItemsController()
    var browser = document.getElementById("redirect-blank");
    browser.init();    
    // setup the stackviewer
    stackOnLoad()
}


function debugger_onUnLoad()
{
    var _gObserverSvc = Components.classes["@mozilla.org/observer-service;1"].
                   getService(Components.interfaces.nsIObserverService);
    _gObserverSvc.removeObserver(DBG.terminate_observer, "invocation_terminated");
    DBG.terminate_observer = null;
    DBG.controller.destroy();
    DBG.controller = null;
    
    stackOnUnload();
}


////////////////////////////////////////////////////////////////
// State related functions 
////////////////////////////////////////////////////////////////

function debugger_change_ui_state(new_state) {
    var view = gViewMgr.currentView; //NOT the current focused, but the current Editor View
    if (!view) return;
    var iv = null;
    try {
        if (view.alternateViewList)
            iv = view.alternateViewList.currentView;
    } catch(e) {
        DBG.log.debug(e);
    }
  
    var viewHasMarkers = view && typeof(view.markerNext) != 'undefined';
    var ivHasMarkers = iv && typeof(iv.markerNext) != 'undefined';

    var old_state = DBG.session.ui_state;
    //DBG.log.debug("debugger_change_ui_state: " + _debugger_stateNames[DBG.session.ui_state] + " => " + _debugger_stateNames[new_state] + "\n");
    
    // The only thing that goes wrong with this optimization is that we
    // may miss a "running" state if we hit 2 "break" states quickly in
    // succession.  As this doesnt think we have moved, the breakpoint
    // stays at the original position.
    // Also can not afford to miss a stop (even though we never should)
    // So only skip intermediate "run" states.
    if (DBG.session.ui_state == new_state &&
        new_state != DBG.STATE_BREAK && new_state != DBG.STATE_STOPPED) {
        //dump("skipping state update with " + new_state + "\n");
        window.updateCommands("debug_state");
        return;
    }

    // XXX is this really necessary?
    if (new_state == DBG.STATE_STOPPED) {
        document.getElementById("menu_dbgGo").setAttribute("label", 'Start');
    } else if (new_state == DBG.STATE_BREAK) {
        document.getElementById("menu_dbgGo").setAttribute("label", 'Continue');
    }

  // Remember the state now.
    DBG.session.ui_state = new_state;
    switch(new_state) {
    case DBG.STATE_STOPPED:
        StatusBar_AddMessage("Debugging session has ended.", "debug",
                             3000, false);
        if (view && viewHasMarkers) {
            view.markerDeleteAll(Editor_Markers.cl);
            view.markerDeleteAll(Editor_Markers.clb);
        }
        if (iv && ivHasMarkers) {
            iv.markerDeleteAll(Editor_Markers.cl);
            iv.markerDeleteAll(Editor_Markers.clb);
        }
        DBG.session.terminal.endSession();

        if (DBG.session && DBG.session.dbg) {
            DBG.session.reset();
            DBG.listener_manager.stopListening();
        }
        uilayout_updateTitlebar(view);
        window.setCursor("auto");
        break;

    case DBG.STATE_STARTING:
        StatusBar_AddMessage("Debugger is starting...", "debug", 0, false);
        DBG.session.viewAtDebuggerStart = view;
        DBG.session.lineAtDebuggerStart = view.currentLine;
        DBG.session.anchorAtDebuggerStart = view.anchor;
        DBG.session.currentColumnAtDebuggerStart = view.currentColumn;
        break;
        
    case DBG.STATE_RUNNING:
        StatusBar_AddMessage("Debugger is running...", "debug", 0, false);
        if (view && viewHasMarkers) {
            view.markerDeleteAll(Editor_Markers.cl);
            view.markerDeleteAll(Editor_Markers.clb);
        }
        if (iv && ivHasMarkers) {
            iv.markerDeleteAll(Editor_Markers.cl);
            iv.markerDeleteAll(Editor_Markers.clb);
        }
        uilayout_updateTitlebar(view);
        break;

    case DBG.STATE_BREAK:
        // Move to the location of the break.
        // XXX - later another debugger listener (such as the project
        // manager) should probably do this.

        var temp = new Object();
        temp.value = null;
        try {
            DBG.session.dbg.enumStackFrames(temp, new Object());
        } catch(e) {
            DBG.log.debug(e);
        }
        var stack = temp.value;
        if (!stack || stack.length < 1 || !stack[0]) {
            window.updateCommands("debug_state");
            try {
                DBG.session.dbg.resume(Components.interfaces.koIDebugger.RESUME_STEP_OVER);
            } catch(e) {
                DBG.log.debug(e);
            }
            return;
        }
        var stack_frame = stack[0].QueryInterface(Components.interfaces.koIDebugStackFrame);
        
        // get the new view for the stack frame, otherwise breakpoints and title bar
        // do not get updated correctly
        view = debugger_show_stack_position(stack_frame, 0);
        iv = null;
        try {
            if (view && view.alternateViewList)
                iv = view.alternateViewList.currentView;
        } catch(e) {
            DBG.log.debug(e);
        }

        var breakLocation = "Break at " + uriparse_displayPath(stack_frame.uri)
                            + ", line " + stack_frame.lineNumber + ".";
        StatusBar_AddMessage(breakLocation, "debug", 0, false);
        uilayout_updateTitlebar(view);
        break;

    case DBG.STATE_STOPPING:
        StatusBar_AddMessage("Debugger is stopping...", "debug", 0, false);
        break;

    } // end switch(new_state)
    // Breakpoints are always updated
    // (and currently need to be done after we change locations)
    debugger_update_breakpoints(view);
    if (iv) debugger_update_breakpoints(iv,view); // update breakpoints for input view
    window.updateCommands("debug_state");
    //DBG.log.debug("debugger_change_ui_state OUT : " + _debugger_stateNames[DBG.session.ui_state] + " (really) " + _debugger_stateNames[DBG.session.dbg.state]);

    if (new_state == DBG.STATE_STOPPED) {
        try {
            // If the view that we started with still exists (wasn't closed), get back to it.
            DBG.session.viewAtDebuggerStart.makeCurrent();
            DBG.session.viewAtDebuggerStart.currentLine = DBG.session.lineAtDebuggerStart;
            DBG.session.viewAtDebuggerStart.anchor = DBG.session.anchorAtDebuggerStart;
            DBG.session.viewAtDebuggerStart.currentColumn = DBG.session.currentColumnAtDebuggerStart;
        } catch (e) {
            // none of the above really matters.
            // DBG.log.debug(e);
        }
    }
}

function debugger_show_current_statement() {
    var temp = new Object();
    DBG.session.dbg.enumStackFrames(temp, new Object());
    var stack = temp.value;
    var stack_frame = stack[0].QueryInterface(Components.interfaces.koIDebugStackFrame);
    
    // DBG.log.debug("uri is '" + stack_frame.uri + "', file is " +
    // stack_frame.filename + ", line " + stack_frame.lineNumber + "\n");
    debugger_show_stack_position(stack_frame, 0); 
}

function debugger_reset_stack_position(view) {
    var fv = gViewMgr.currentView;
    if (fv && !view != fv && fv.alternateViewList) {
        if (view == fv.alternateViewList.currentView) {
            /* when we reset the parent pane, this pane
               will get reset also */
            return;
        }
    }

    var viewHasMarkers = view && typeof(view.markerNext) != 'undefined';
    
    if (viewHasMarkers) {
        view.markerDeleteAll(Editor_Markers.cl);
        view.markerDeleteAll(Editor_Markers.clb);
    }
    var iv = null;
    if (view.alternateViewList)
        iv = view.alternateViewList.currentView;
    var ivHasMarkers = iv && typeof(iv.markerNext) != 'undefined';
    
    if (DBG.session.ui_state == DBG.STATE_STOPPED) return;
    if (DBG.session.currentmarker.file == view.document.file.URI) {
        if (viewHasMarkers) {
            view.markerAdd(DBG.session.currentmarker.line, Editor_Markers.cl);
            view.markerAdd(DBG.session.currentmarker.line, Editor_Markers.clb);
        }
        if (ivHasMarkers && DBG.session.currentmarker.ifile && iv && iv.document.file.URI == DBG.session.currentmarker.ifile) {
            iv.markerDeleteAll(Editor_Markers.cl);
            iv.markerDeleteAll(Editor_Markers.clb);
            iv.markerAdd(DBG.session.currentmarker.iline, Editor_Markers.cl);
            iv.markerAdd(DBG.session.currentmarker.iline, Editor_Markers.clb);
        }
    }
}

function debugger_show_stack_position(stackFrame, frameNumber) {
  try {
    // If we're doing an eval in Perl, it isn't a real position.
    // So we aren't going to show it, and the perl debugger will
    // keep going as well.
    if (stackFrame.filename.match(/\(eval\s+\d+\)\[/)) {
      // Whatever we were doing before we hit the eval, keep doing it.
      // XXX set up a temporary buffer to hold the contents of the eval
      if (DBG.session.lastCommand) {
        DBG.session.dbg.resume(DBG.session.lastCommand);
      }
      return null;
    }

    DBG.session.currentmarker = new DBG_CurrentMarker();
    
    // XXX Fixme : need to do smarter koResource stuff here.
    // Just attempt to open the URI.  If it is already open, then fine!
    DBG.log.info('StackFrame URI: '+stackFrame.uri+' Line: '+stackFrame.lineNumber);
    var v = debugger_doFileOpenAtLine(stackFrame.uri, stackFrame.lineNumber);
    var schemeService = Components.classes['@activestate.com/koScintillaSchemeService;1'].getService();
    var scheme = schemeService.getScheme(gPrefs.getStringPref('editor-scheme'));
    var viewHasMarkers = v && typeof(v.markerNext) != 'undefined';
    if (v) {
        var linecolor = hexColorToLong(scheme.getColor('callingLineColor'));
        var linecolorlight = hexColorToLong(scheme.getColor('currentLineColor'));
        if (viewHasMarkers) {
            v.markerDeleteAll(Editor_Markers.cl);
            v.markerDeleteAll(Editor_Markers.clb);
    
            if (frameNumber != 0) {
                v.scimoz.markerSetBack(Editor_Markers.clb, linecolor);
            } else {
                v.scimoz.markerSetBack(Editor_Markers.clb, linecolorlight);
            }
    
            v.markerAdd(stackFrame.lineNumber-1, Editor_Markers.cl);
            v.markerAdd(stackFrame.lineNumber-1, Editor_Markers.clb);
        }
        DBG.session.currentmarker.file = v.document.file.URI;
        DBG.session.currentmarker.line = stackFrame.lineNumber-1;
  
        // XXX attempt to open input file
        // this is temporary until new views work is done, will only effect ActiveXSLT debugging
        if (typeof(stackFrame.inputURI) != 'undefined' && stackFrame.inputURI != "") {
            DBG.log.info("INPUT FILE: "+stackFrame.inputURI+" LINE: "+stackFrame.inputLineNumber+"\n");
            try {
                if (v.alternateViewList) {
                    var views = v.alternateViewList.findViewsForURI(stackFrame.inputURI);
                    var iv = null;
                    if (views.length) {
                        iv = views[0];
                        if (iv != v.alternateViewList.currentView)
                            iv.makeCurrent(true); // dont grab focus
                        iv.currentLine = stackFrame.inputLineNumber;
                    } else {
                        iv = debugger_doFileOpenAtLine(stackFrame.inputURI,stackFrame.inputLineNumber,null,v.alternateViewList);
                        v.setFocus();
                    }
                    if (iv) {
                        var ivHasMarkers = typeof(iv.markerNext) != 'undefined';
                        if (ivHasMarkers) {
                            iv.markerDeleteAll(Editor_Markers.cl);
                            iv.markerDeleteAll(Editor_Markers.clb);
        
                            if (frameNumber != 0) {
                                iv.scimoz.markerSetBack(Editor_Markers.clb, linecolor);
                            } else {
                                iv.scimoz.markerSetBack(Editor_Markers.clb, linecolorlight);
                            }
        
                            iv.markerAdd(stackFrame.inputLineNumber-1, Editor_Markers.cl);
                            iv.markerAdd(stackFrame.inputLineNumber-1, Editor_Markers.clb);
                        }                        
                        DBG.session.currentmarker.ifile = iv.document.file.URI;
                        DBG.session.currentmarker.iline = stackFrame.inputLineNumber-1;
                    }
                }
            } catch(e) { DBG.log.exception(e); }
        }
    }
    return v;
  } catch (e) {
    log.exception(e);
  }
  return null;
}

function debugger_find_bp(dbg, url, lineNo) {
    var ienum = dbg.enumBreakpoints(url);
    var bp;
    while (ienum.hasMoreElements()) {
        bp = ienum.getNext();
        bp = bp.QueryInterface(Components.interfaces.koIBreakpoint);
        if (bp.lineNumber == lineNo) {
            return bp;
        }
    }
    return null;
}


function debugger_update_breakpoints(view,pv) {
    // Synchronizes the breakpoints view of the world for the debugger and the view
    // no point in updating breakpoints if there are no open files
    if (view == null) {
        return;
    }
    if (typeof(pv)=='undefined') pv=null;
    
    // if our vew is the inputview, and we didn't get
    // the parent, get it now
    var fv = gViewMgr.currentView;
    if (fv && !pv && view != fv && fv.alternateViewList) {
        if (view == fv.alternateViewList.currentView) {
            pv = fv;
        }
    }
    // if the view does not support markers, skip it
    if (typeof(view.markerNext) == 'undefined') {
        return;
    }
    // reach into debugger for full name
    var url = view.document.file.URI;

    var marker_mask = 1 << Editor_Markers.wb | 1 << Editor_Markers.fb;
    var lineNo = -1;
    var bp, markertype;
    // Make sure that all of the breakpoints in scimoz exist in the debugger
    while (true) {
        lineNo = view.markerNext(lineNo+1, marker_mask);
        if (lineNo < 0)
            break;
        //breakpoints.appendLongPref(lineNo);
        bp = debugger_find_bp(DBG.session.dbg, url, lineNo+1);
        if (!bp) {
            var lang = pv?pv.document.language:view.document.language;
            DBG.session.dbg.addBreakpoint(lang, view.document.file.URI, lineNo+1);
        } else {
            // make sure that the debugger breakpoint state determines
            // the viewpoint breakpoint type (red vs. green)
            markertype = view.markerGet(lineNo);
            var markerPending = ! (markertype & (1 << Editor_Markers.wb));
            if (bp.state == Components.interfaces.koIBreakpoint.BREAKPOINT_DELETED) {
                bp.state = Components.interfaces.koIBreakpoint.BREAKPOINT_ENABLED;
                view.markerAdd(lineNo, (DBG.session.dbg.state == DBG.STATE_BREAK)?Editor_Markers.wb:Editor_Markers.fb);
            } else if (bp.state == Components.interfaces.koIBreakpoint.BREAKPOINT_DISABLED) {
                bp.state = Components.interfaces.koIBreakpoint.BREAKPOINT_DELETED;
                view.markerDelete(lineNo, markerPending?Editor_Markers.fb:Editor_Markers.wb);
            } else if (bp.pending && ! markerPending) {
                view.markerDelete(lineNo, Editor_Markers.wb);
                view.markerAdd(lineNo, Editor_Markers.fb);
            }
            else if (! bp.pending && markerPending) {
                view.markerDelete(lineNo, Editor_Markers.fb);
                view.markerAdd(lineNo, Editor_Markers.wb);
            }
        }
    }
    // make sure that all of the breakpoints in the debugger correspond to scimoz
    // breakpoints
    var ienum = DBG.session.dbg.enumBreakpoints(url);
    while (ienum.hasMoreElements()) {
        bp = ienum.getNext();
        bp = bp.QueryInterface(Components.interfaces.koIBreakpoint);
        markertype = view.markerGet(bp.lineNumber-1);
        if (! (markertype & ( 1 << Editor_Markers.wb)) &&
            ! (markertype & ( 1 << Editor_Markers.fb))) {
            bp.state = Components.interfaces.koIBreakpoint.BREAKPOINT_DELETED;
        }
    }
}

function debugger_deleteBreakpointList(url) {
    var ienum = DBG.session.dbg.enumBreakpoints(url);
    var bp;
    while (ienum.hasMoreElements()) {
        bp = ienum.getNext();
        bp = bp.QueryInterface(Components.interfaces.koIBreakpoint);
        bp.state = Components.interfaces.koIBreakpoint.BREAKPOINT_DELETED;
    }
}

function debugger_toggle_breakpoint(view, lineClicked) {
    if (typeof(view) == 'undefined' || !view) {
        view = gViewMgr.currentView;
        if (view)
            lineClicked = view.currentLine - 1;
    }
    if (!view) {
        DBG.log.error("toggle-breakpoints: internal error: can't access the current buffer");
        return;
    }
    var bp = debugger_find_bp(DBG.session.dbg, view.document.file.URI, lineClicked+1);
    var markertype = view.markerGet(lineClicked);
    if (markertype & (1 << Editor_Markers.wb)) {
        view.markerDelete(lineClicked, Editor_Markers.wb);
        if (bp) bp.state = Components.interfaces.koIBreakpoint.BREAKPOINT_DELETED;
    }
    if (markertype & (1 << Editor_Markers.fb)) {
        view.markerDelete(lineClicked, Editor_Markers.fb);
        if (bp) bp.state = Components.interfaces.koIBreakpoint.BREAKPOINT_DELETED;
    }
    if (! (markertype & (1 << Editor_Markers.wb)) &&
        ! (markertype & (1 << Editor_Markers.fb))) {
        if (bp) bp.state = Components.interfaces.koIBreakpoint.BREAKPOINT_ENABLED;
        if (DBG.session.dbg.state == DBG.STATE_BREAK) {
            view.markerAdd(lineClicked, Editor_Markers.wb);
        } else {
            view.markerAdd(lineClicked, Editor_Markers.fb);
        }
    }
    debugger_update_breakpoints(view);
}


function debugger_clear_breakpoints () {
    var v = gViewMgr.currentView;
    if (!v) return 0;
    v.markerClearAll(Editor_Markers.fb);
    v.markerClearAll(Editor_Markers.wb);
    debugger_deleteBreakpointList(v.document.file.URI);
    return 1;
}

// This is used for internal use: passing arbitrary commands
// to the debugger, before instituting the functionality into
// regular UI elements.


////////////////////////////////////////////////////////////////
// UI callback do_* event handlers.
////////////////////////////////////////////////////////////////

//function debugger_do_toggle_breakpoint () {
//    var v = gViewMgr.currentView;
//    if (!v) return;
//  
//    var lineClicked = v.scimoz.lineFromPosition(v.scimoz.currentPos);
//    debugger_toggle_breakpoint(v, lineClicked);
//}

function debugger_do_step(step_type) {
    var v = gViewMgr.currentView;
    if (!v) return;
    if (!debugger_check_start(step_type)) {
        DBG.log.debug("debugger_do_step: debugger_check_start failed, either because user cancelled dialog or because something is wrong w/ debugger engine registration\n");
        return;
    }
    DBG.session.lastCommand = step_type;
    var bp = null;
    
    try {
        var iv = null;
        if (v.alternateViewList) {
            iv = v.alternateViewList.currentView;
            if (iv)
                debugger_update_breakpoints(iv,v);
        } else {
            debugger_update_breakpoints(v);
        }
    } catch(e) {
        DBG.log.debug(e);
    }

    // this handles 'run-to-cursor' capabilitiy.  It should always
    // be dependent on the ctrl key being pressed.  The idea is that
    // we set a temporary breakpoint on the line where the cursor is,
    // then we change our step type to GO.  when we come out of
    // dbg.resume below, we unset the temporary breakpoint.
    if (v && DBG.controller.ctrlKeyForStart &&
        step_type == Components.interfaces.koIDebugger.RESUME_STEP_OVER) {
        // set breakpoint
        bp = DBG.session.dbg.addBreakpoint(view.document.language, v.document.file.URI, v.currentLine);
        step_type = Components.interfaces.koIDebugger.RESUME_GO
        if (bp == null) {
            alert("Can not stop at this location.");
            return;
        }
    }
    
    if (DBG.session.dbg.state == DBG.STATE_BREAK) {
        try {
            DBG.session.dbg.resume(step_type);
        } catch(e) {
            DBG.log.debug("XXX Caught exception on DBG.session.dbg.resume\n");
        }
    }
  
    // clean up the temporary breakpoint that was added for
    // run-to-cursor
    if (bp && bp.state == Components.interfaces.koIBreakpoint.BREAKPOINT_ENABLED) {
        bp.state = Components.interfaces.koIBreakpoint.BREAKPOINT_DELETED;
    }
}

// url may be null indicating no redirection for this run.
function debugger_resetTerminal(show_terminal, emulate_cgi) {
    // XXX - Hack alert.  Mozilla may reuse our plugins.  Ensure that our
    // main scimoz widget is made visible before us.
    var swstack = document.getElementById("redirect-stack");
    swstack.setAttribute("selectedIndex", "1");

    // switch to the run output tab
    var tabsWidget = document.getElementById("output_tabs");
    var output_tab = document.getElementById("output_tab");
    var var_tab = document.getElementById("vars_tab");
    if (tabsWidget.selectedItem != var_tab)
        tabsWidget.selectedItem = output_tab;

    if (!DBG.session.terminal.active)
        DBG.session.initTerminal();
}

var gOpeningDialog = false;

// Get the "current" invocation to use, creating an appropriate one of necessary.
// Need some smarts here:
// A "project" and a "view" optionally have invocation data
// If the current view has a set, we use them.
// If not, look if the project has one, and use them.
// Depending on requirements, each project/file may have multiple
// invocations stored - so may need to pass multiple invocations to the
// diaDBG.log.
function _debugger_do_get_invocation() {
    // MUST get the MAIN viewManagers current view for debugging
    var curView =  gViewMgr.currentView;
    if (curView == null || !curView.document || !curView.document.file)
        return null;
    var skipDialog;
    var skipDialogPref = gPrefs.getBooleanPref('debugSkipDialog');
    if (skipDialogPref) {
        skipDialog = !DBG.controller.ctrlKeyForStart;
    } else {
        skipDialog = DBG.controller.ctrlKeyForStart;
    }
    var prefs = getEffectivePrefsForFile(curView.document.file.URI);
    var langName = curView.document.language;

    DBG.session.filename = "";
    DBG.session.input_filename = "";
    try {
        DBG.session.filename = curView.document.file.path;
        // XXX a better way to handle input views is necessary!
        if (langName == "XSLT" && curView.alternateViewList) {
            var iv = curView.alternateViewList.currentView;
            if (iv) DBG.session.input_filename = iv.document.file.displayPath;
        }        
    } catch (e) {
        // Invalid URL - Quite likely a "kodebugger" URL!
        // that is fine - we still allow them to invoke something, just don't
        // default the filename.
    }

    // Locate an invocation to use.
    var invocation = null;
    var def_invocation_name = langName;
    
    // Create a default invocation for this view.
    if (langName && def_invocation_name) {
        var invocation_svc = Components.classes["@activestate.com/InvocationRegistryService;1"].getService();
        var info = invocation_svc.getInvocationInformation(def_invocation_name);
        if (info) {
            try {
                invocation = Components.classes[info.contractid].createInstance();
            } catch (e) {
                DBG.log.error("Failed to create invocation '" + info.contractid + "' - " + e + "\n");
                invocation = null;
            }
        }
        if (invocation) {
            var catOb = new Object();
            var instOb = new Object();
            invocation.getDefaultPreferences(catOb, instOb);
            invocation.currentCategoryPreferences = catOb.value;
            invocation.currentInstancePreferences = instOb.value;
            var storedPrefs,root;
            
            // clear any global prefs for the filename
            // to prevent pref chaining the wrong filename
            if (gPrefs.hasPref("Invocations")) {
                root = gPrefs.getPref("Invocations");
                if (root.hasPref(invocation.name)) {
                    storedPrefs = root.getPref(invocation.name);
                    storedPrefs.setStringPref("filename","");
                    storedPrefs.setStringPref("inputfile","");
                }
            }
            
            var iprefs = invocation.currentInstancePreferences;
            // Update the invocation preferences from the global preference set.
            if (prefs.hasPref("Invocations")) {
                root = prefs.getPref("Invocations");
                if (root.hasPref(invocation.name)) {
                    storedPrefs = root.getPref(invocation.name);
                    iprefs.update( storedPrefs );
                }
            }
            
            
            // Setup other default values.
            if (iprefs.hasStringPref("filename") &&
                DBG.session.filename &&
                !iprefs.getStringPref("filename")) {
                iprefs.setStringPref("filename", DBG.session.filename);
            }
            if (iprefs.hasStringPref("inputfile") &&
                DBG.session.input_filename &&
                !iprefs.getStringPref("inputfile")) {
                iprefs.setStringPref("inputfile", DBG.session.input_filename);
            }
        }
    }

    supports_debug = (invocation.supportedInvokeTypes & DBG.INVOKE_DEBUG) != 0;
    
    if (DBG.session.invoke_type == DBG.INVOKE_DEBUG && !supports_debug) {
        dialog_alert('Please correct your interpreter selection in preferences.', null, 'Debugger Error');
        return null;
    }
    // If I don't have an invocation, or it is not valid, I must show the diaDBG.log.
    // XXX - it would be nice if this preference was per invocation, but this begs
    // the problem - how to enable it per invocation if that invocation's dialog isn't
    // shown.  Must wait for an "Invocation Preferences" UI element on the file/project itself
    if (skipDialog && (!invocation || !invocation.validate(DBG.session.invoke_type, new Object(), new Object())))
        skipDialog = false;

    if (!skipDialog) {
        var resp = new Object ();
        resp.prefs = prefs;
        resp.invocation = invocation;
        resp.invoke_type = DBG.session.invoke_type;
        resp.res = "";
        // Check to see if there's a debugger dialog being opened
        // Not quite sure why this happens only for this dialog, but without
        // this guard, hitting F10 and F11 simultaneously results in a crash.
        if (gOpeningDialog) {
            return null;
        }
        gOpeningDialog = true;
        window.openDialog
                 ("chrome://jaguar/content/pref/invocations/debugging-properties.xul",
                  "Komodo",
                  "chrome,dialog=yes,close=yes,dependent=yes,modal=yes",
                  resp );
        if (resp.res != "ok" || resp.invocation==null) {
            //DBG.log.debug("User cancelled debugging args dialog");
            return null;
        }
        invocation = resp.invocation;
        // And save the invocation properties back.
        if (!prefs.hasPrefHere("Invocations")) {
            var newPrefs = Components.classes["@activestate.com/koPreferenceSet;1"].createInstance();
            newPrefs.id = "Invocations";
            prefs.setPref( newPrefs.id, newPrefs);
        }
        var prefRoot = prefs.getPref("Invocations");
        prefRoot.parent = prefs;
        // XXX - this is really a workaround for bug 18218, and should probably die,
        // using invocation.instancePreferences
        var invocation_prefs;
        if (prefRoot.hasPref(invocation.name)) {
            invocation_prefs = prefRoot.getPref(invocation.name);
            invocation_prefs.parent = prefRoot;
            invocation_prefs.update(invocation.persistableInstancePreferences);
        } else {
            invocation_prefs = invocation.persistableInstancePreferences
            invocation_prefs.parent = prefRoot
            prefRoot.setPref(invocation.name, invocation_prefs);
        }
    }
    if (!invocation) {
        DBG.log.error("Shouldn't have managed to get this far without an invocation or the dialog being cancelled");
        return null;
    }
    return invocation;
}

// Called by the listener when it has a debugger connection.
function debugger_on_connection(language, host, port, cookie)
{
    var invocation = DBG.session.invocation;
    var step_type = DBG.session.initial_step_type;
    DBG.session.initial_step_type = Components.interfaces.koIDebugger.RESUME_STEP_IN;

    // XXX - move to caller - pass the prefset
    // Create a preference set for the debugger.
    prefset = Components.classes["@activestate.com/koPreferenceSet;1"].createInstance();
    prefset.setStringPref("debugger.listener-reponse.cookie", cookie);
    prefset.setStringPref("debugger.language", language);
    prefset.setStringPref("debugger.listener-reponse.host", host);
    prefset.setLongPref("debugger.listener-reponse.port", port);
    var stdin = null;
    var stdout = null;
    var stderr = null;
    var show_terminal;
    var emulate_cgi = false;
    var debugger_redirect = false;
    if (!_debugger_is_local_host(host)){
        // remote connection - redirect IO via the debugger.
        debugger_redirect = true;
        show_terminal = true;
    } else {
        // local connection - if we have an invocation, then use the "use-console"
        // preference - otherwise treat as remote debug.
        if (invocation) {
            // hook up invocation preferences, and use them for debug session.
            invocation.currentCategoryPreferences.parent = prefset;
            prefset = invocation.currentInstancePreferences;
            debugger_redirect = false;
            show_terminal = !prefset.hasBooleanPref("use-console") || !prefset.getBooleanPref("use-console");
            emulate_cgi = !prefset.hasBooleanPref("sim-cgi") || !prefset.getBooleanPref("sim-cgi");
        } else {
            // local, but not via invocation - treat as remote.
            debugger_redirect = true;
            show_terminal = true;
        }
    }
    debugger_resetTerminal(show_terminal, emulate_cgi);
    if (debugger_redirect) {
        if (!show_terminal) DBG.log.error("Logic Error - redirecting debugger output, but not showing terminal!");
        stdin = DBG.session.terminal.stdin;
        stdout = DBG.session.terminal.stdout;
        stderr = DBG.session.terminal.stderr;
    }
    DBG.session.dbg.startDebugging(step_type, prefset, stdin, stdout, stderr);
}

function _debugger_do_execute_invocation(invocation, invoke_type, new_debugger_state) {
    var use_console = invocation.currentInstancePreferences.getBooleanPref("use-console");
    var emulate_cgi = invocation.currentInstancePreferences.hasPref("sim-cgi") ?
            invocation.currentInstancePreferences.getBooleanPref("sim-cgi") :
            false;

    debugger_resetTerminal(!use_console, emulate_cgi);
    if (!use_console) {
        if (!emulate_cgi)
            invocation.stdin = DBG.session.terminal.stdin;
        invocation.stdout = DBG.session.terminal.stdout;
        invocation.stderr = DBG.session.terminal.stderr;
    }
    // and fire off the process.
    var rc;
    try {    
        invocation.invoke(invoke_type);
        if (new_debugger_state != null)
            debugger_change_ui_state(new_debugger_state)
        rc = true;
    } catch (e) {
        debugger_change_ui_state(DBG.STATE_STOPPED);
        rc = false;
        DBG.log.error("There was an unexpected error starting the invocation:" + e);
    }
    return rc;
}

function _debugger_do_start_debugger(initial_step_type, invoke_type) {
    if (!gViewMgr.offerToSave(null, /* urls */
                              'Save Modified Files?', /* title */
                              'Save Modified Files Before Debugging', /* prompt */
                              "save_on_debug", /* pref */
                              true /* skip projects */
                              )) {
        // user canceled
        return false;
    }

    DBG.session.invoke_type = invoke_type;
    var invocation = _debugger_do_get_invocation();
    if (invocation == null)
        return false;
    DBG.session.invocation = invocation;
    DBG.session.initial_step_type = initial_step_type;
    // XXX - maybe we should abort here, telling the user
    supports_debug = (invocation.supportedInvokeTypes & DBG.INVOKE_DEBUG) != 0;
    if (invoke_type == DBG.INVOKE_DEBUG && !supports_debug) {
        dialog_alert('We do not support debugging for this file. '+
                     'If you feel this is in error, you may change your file '+
                     'associations in your preferences.', null, 'Debugger Error');
        return false;
    }
    DBG.session.dbg.state = DBG.STATE_STARTING;
    debugger_change_ui_state(DBG.STATE_STARTING);
    
    // start the listener, and allow it 10 seconds before it shows itself.
    // XXX when persistent listening is done, this needs to be removed
    if (invoke_type == DBG.INVOKE_DEBUG)
        DBG.listener_manager.startListening(10000, true);

    var new_state = invoke_type == DBG.INVOKE_DEBUG ? DBG.STATE_RUNNING : null;
    // and fire off the process.
    var ok =  _debugger_do_execute_invocation(invocation, invoke_type, new_state)

    if (!ok) {
        if (invoke_type == DBG.INVOKE_DEBUG) {
            try {
                DBG.listener_manager.stopListening();
            } catch (e) {
                DBG.log.error("Error stopping the debugger listener after debugger failure:" + e);
            }
        }
        alert("There was an unexpected error starting the debugger.");
        return false;
    }
    return true;
}

function debugger_check_start(initial_step_type, invoke_type){
    if (DBG.listener_manager.isListening()) {
        // our controllers don't have time to update properly if you
        // click very fast.  So it is possible to be already listening
        // when we get here.  Fail silently.
        return false;
    }
    if (typeof(invoke_type) == 'undefined' || !invoke_type) {
        invoke_type = DBG.INVOKE_DEBUG;
    }
    if (DBG.session.dbg.state == DBG.STATE_STOPPED || DBG.session.dbg.state == DBG.STATE_STARTING) {
        if (DBG.session.ui_state != DBG.STATE_STOPPED) {
            DBG.log.debug("** Detected potential re-entrancy (ui_state is " + DBG.session.ui_state + ") - ignoring debugger request\n");
            return 1; // pretend it worked OK - presumably another debug session _is_ starting correctly and the user is impatient
        }
        return _debugger_do_start_debugger(initial_step_type, invoke_type);
    }

    // return 1, not 0, since everything's working.
    return 1;
}

function debugger_run_script() {
    return debugger_check_start(Components.interfaces.koIDebugger.RESUME_GO,
                                DBG.INVOKE_RUN)
}

/**
 * take the text from the debugger output scintilla, put it into the iframe, and switch
 */
var _debugger_outputTmpFile=null;
              
function debugger_renderOutputAsHTML() {
    var swstack = document.getElementById("redirect-stack");
    var scout = document.getElementById("redirect-scin");
    var browser = document.getElementById("redirect-blank");
    if (_debugger_outputTmpFile) {
        delete _debugger_outputTmpFile;
        _debugger_outputTmpFile = null;
    }
    if (swstack.selectedIndex == 0) {
        swstack.selectedIndex = 1;
    } else {
        try {
            var osPathSvc = Components.classes["@activestate.com/koOsPath;1"].
                      getService(Components.interfaces.koIOsPath);
            var docSvc = Components.classes['@activestate.com/koDocumentService;1']
                        .getService(Components.interfaces.koIDocumentService);
            
            var basedir = null;
            if (DBG.session.filename) basedir = osPathSvc.dirname(DBG.session.filename);
            
            // get the text from scintilla, stuff it into the iframe, and switch the deck
            var text = scout.scimoz.text;
            if (text) {
                var tmpFileSvc = Components.classes["@activestate.com/koFileService;1"] 
                                 .getService(Components.interfaces.koIFileService)
    
                if (basedir) {
                    _debugger_outputTmpFile = tmpFileSvc.makeTempFileInDir(basedir,".html","w");
                } else {
                    _debugger_outputTmpFile = tmpFileSvc.makeTempFile(".html","w");
                }
    
                _debugger_outputTmpFile.puts(text);
                _debugger_outputTmpFile.flush();
                _debugger_outputTmpFile.close();
                var doc = docSvc.createDocumentFromURI(_debugger_outputTmpFile.URI);
                browser.initWithDocument(doc, false);
                swstack.selectedIndex = 0;
            }
        } catch(e) {
            log.exception(e);
        }
    }
}

function debugger_doFileOpenAtLine(uri,linenum,type,viewlist) {
    if (gPrefs.getBooleanPref('debuggerFindLocalFile')) {
        var fileSvc = Components.classes["@activestate.com/koFileService;1"].
                        getService();
        var debuggerURI = Components.classes['@activestate.com/koDebuggerURI;1'].
                        createInstance(Components.interfaces.koIDebuggerURI);
        local_uri = debuggerURI.getBaseURI(uri);
        var file = fileSvc.getFileFromURI(local_uri);
        if (file.exists) {
            uri = file.URI;
        }
    }
    return gViewMgr.doFileOpenAtLine(uri,linenum,type,viewlist);
}

// Test code
function debugger_test_invocation() {
   var obj = new Object ();
   var response = window.openDialog
         ("chrome://jaguar/content/test/test_standalone_invocation.xul",
          "Komodo",
          "chrome,dialog=yes,close=yes,dependent=yes,modal=yes",
          obj)

   var command = obj.command;
   var params = obj.params;

    if (!command)
       return;
    var invocation = Components.classes["@activestate.com/koInvocation;1?type=Perl"].createInstance();
    var catOb = new Object();
    var instOb = new Object();
    invocation.getDefaultPreferences(catOb, instOb);
    invocation.currentCategoryPreferences = catOb.value;
    invocation.currentInstancePreferences = instOb.value;
    var iprefs = invocation.currentInstancePreferences;
    iprefs.setStringPref("filename", command);
    iprefs.setStringPref("params",  params);
    iprefs.setBooleanPref("use-console", false);

    var error_pref = new Object();
    var error_text = new Object()
    if (!invocation.validate(Components.interfaces.koIInvocation.DBG.INVOKE_RUN, error_pref, error_text)) {
        alert("The invocation is not valid - " + error_text.value);
        return;
    }
    _debugger_do_execute_invocation(invocation, Components.interfaces.koIInvocation.DBG.INVOKE_RUN, Components.interfaces.koIDebugger.DBG.STATE_RUNNING);
}
