/* Copyright (c) 2000-2003 ActiveState Corp.
   See the file LICENSE.txt for licensing information. */

/* Dialog to edit an environment variable.
 *
 * Usage:
 *  All dialog interaction is done via an object passed in and out as the first
 *  window argument: window.arguments[0]. All these arguments are optional.
 *      .name           default name for env. var.
 *      .value          default value for env. var.
 *      .title          the dialog title
 *      .mruName        if set this will be used to identify an MRU preference
 *                      set and an MRU will be provided
 *  On return window.arguments[0] has:
 *      .retval         "OK" or "Cancel" indicating how the dialog was exitted
 *  and iff .retval == "OK":
 *      .name           the name to user entered
 *      .value          the value to user entered
 *
 */

//---- globals

var _gUsingMRU = false;



//---- interface routines for XUL

function OnLoad()
{
    var dialog = document.getElementById("dialog")
    var okButton = dialog.getButton("accept");
    var cancelButton = dialog.getButton("cancel");
    okButton.setAttribute("accesskey", "o");
    cancelButton.setAttribute("accesskey", "c");

    // .name
    var name = window.arguments[0].name;
    if (typeof name == "undefined" || name == null) {
        name = "";
    }
    var nameWidget = document.getElementById("name");
    nameWidget.setAttribute("value", name);

    // .value
    var value = window.arguments[0].value;
    if (typeof value == "undefined" || value == null) {
        value = "";
    }
    var valueWidget = document.getElementById("value");
    valueWidget.setAttribute("value", value);

    // .title
    if (typeof window.arguments[0].title != "undefined" &&
        window.arguments[0].title != null) {
        window.title = window.arguments[0].title;
    } else {
        window.title = "Edit Variable";
    }

    // .mruName
    var mruName = window.arguments[0].mruName;
    if (typeof mruName != "undefined" && mruName != null) {
        // Need to delay this until "komodo-preference-set" is properly set.
        var session;
        nameWidget.setAttribute("komodo-preference-set", mruName+"_name_mru");
        session = new MRUAutoCompleteSession(nameWidget);
        nameWidget.setAttribute("_session", session);
        valueWidget.setAttribute("komodo-preference-set", mruName+"_value_mru");
        session = new MRUAutoCompleteSession(valueWidget);
        valueWidget.setAttribute("_session", session);
        _gUsingMRU = true;
    }

    window.sizeToContent();
    if (opener.innerHeight == 0) { // indicator that opener hasn't loaded yet
        dialog.centerWindowOnScreen();
    } else {
        dialog.moveToAlertPosition(); // requires a loaded opener
    }
    
    if (name) {
        valueWidget.focus();
    } else {
        nameWidget.focus();
    }
}

function OnUnload()
{
    if (typeof window.arguments[0].retval == "undefined") {
        // This happens when "X" window close button is pressed.
        window.arguments[0].retval = "Cancel";
    } else if (window.arguments[0].retval == "OK") {
        var nameWidget = document.getElementById("name");
        window.arguments[0].name = nameWidget.value;
        if (_gUsingMRU) {
            MRU_updateFromWidget(nameWidget);
        }
        var valueWidget = document.getElementById("value");
        window.arguments[0].value = valueWidget.value;
        if (_gUsingMRU) {
            MRU_updateFromWidget(valueWidget);
        }
    }
}


function AddPath()
{



    var pathsep = ":";


    var dname = filepicker_getFolder();
    if (dname) {
        var valueWidget = document.getElementById("value");
        if (!valueWidget.value) {
            valueWidget.value = dname
        } else {
            var value = valueWidget.value;
            if (value[value.length-1] != pathsep)
                value += pathsep
            value += dname;
            valueWidget.value = value
        }
    }
}

function OK()
{
    window.arguments[0].retval = "OK";
    return true;
}


function Cancel()
{
    window.arguments[0].retval = "Cancel";
    return true;
}


