/* Copyright (c) 2000-2002 ActiveState SRL.
   See the file LICENSE.txt for licensing information. */

// Handle drag-drop of files onto the main Workspace window.
// this relies on using nsDragDrop.js

var workspaceDragObserver = {
    canHandleMultipleItems: true,
    _log: null,
    get log() {
        if (!this._log) {
            this._log = getLoggingMgr().getLogger('workspaceDD');
        }
        return this._log;
    },
    focusWindow: function() {
        /*
          This function needs to make the window be active and focused
          to fix problems with dropping objects onto komodo from outside
          applications.
          
          XXX this should work to activeate the window, but
          setFocus is not implemented in nsXULWindow.cpp.  The code
          below is taken partly from filepicker.js in xpfe
          
        var baseWin = null;
        try {
            baseWin = window.QueryInterface(Components.interfaces.nsIInterfaceRequestor)
                            .getInterface(Components.interfaces.nsIWebNavigation)
                            .QueryInterface(Components.interfaces.nsIDocShellTreeItem)
                            .treeOwner
                            .QueryInterface(Components.interfaces.nsIInterfaceRequestor)
                            .getInterface(Components.interfaces.nsIBaseWindow);
            baseWin.setFocus();
        } catch(ex) {
            this.log.exception(ex,'Unable to get base window');
        }
        */
    },
    getSupportedFlavours: function() {
        var flavours = new FlavourSet();
        flavours.appendFlavour("text/unicode");
        flavours.appendFlavour("text/x-moz-url");
        flavours.appendFlavour("text/x-moz-file");

        flavours.appendFlavour("TEXT");

        return flavours;
    },
    canDrop: function(event,session) {
        return true;
    },
    onDragOver: function(event,flavour,session) {
        return true;
    },
    onDrop: function(event, transferDataSet, dragSession) {
        var num_items = 0;
        var uri_open_list = [];
        for ( var i = 0; i < transferDataSet.dataList.length; ++i ) {
            var transferData = transferDataSet.dataList[i];
            var desc = new Object();
            // from DragDrop.js
            unpackDroppedData(transferData.dataList[0], desc)
            desc.dragAction = dragSession.dragAction;
            this.log.debug('workspaceDragObserver data:'+desc.text+'\n');
            if (desc.isFileURL || desc.isFTPURL) {
                this.log.debug('workspaceDragObserver opening files\n');
                uri_open_list.push(desc.text);
                num_items++;
            }
        } // foreach drag item
        if (uri_open_list.length)
            open_openMultipleURIs(uri_open_list);
        if (num_items < 1) {
            return desc;
        }
        // force a focus onto the window after having files dropped on it
        this.focusWindow();
        return null;
    },
    doDragOverEvent: function(event) {
        this.log.debug('workspaceDragObserver got dragOver event');
        try {
            nsDragAndDrop.dragOver(event,this);
        } catch(e) {
            this.log.exception(e,'workspaceDragObserver dragOver exception:');
        }
    },
    doDropEvent: function(event) {
        this.log.debug('workspaceDragObserver got drop event');
        try {
            nsDragAndDrop.drop(event,this);
        } catch(e) {
            this.log.error(e,'workspaceDragObserver drop exception:');
        }
    }
}

function unpackDroppedData(flavourData, ret) {
    var os = Components.classes["@activestate.com/koOs;1"].
          getService(Components.interfaces.koIOs);
    var log = workspaceDragObserver.log;
    var fname = null;
    var url = null;
    ret.text = null;
    ret.flavour = flavourData.flavour.contentType;
    ret.isURL = false;
    ret.isFileURL = false;
    ret.isFTPURL= false;
    ret.isFileOrDir = false;  // true for both file:// and ftp://
    ret.isRemoteFileOrDir = false; // true for ftp:// and koremote://
    ret.isExistingFile = false;
    ret.isDebuggerFile = false;
    ret.isDir = false;
    switch (ret.flavour) {
    case "application/x-moz-file":
        log.debug("We're in x-moz-file?!??\n");
        try { // XXX don't know when this is true ...
            ret.text = flavourData.data.QueryInterface(Components.interfaces.nsIFile).target;
        } catch(e) {
            return;
        }
        ret.isFileOrDir = true;
        ret.isFileURL = true;
        ret.isExistingFile = os.path.exists(ret.text);
        log.debug('type was application/x-moz-file\n');
        return;
        break;
    default: // we try this as a url irregardless of type
        ret.text = flavourData.data;
        log.debug('dropped data is '+ret.text+'\n');
        var origdata = ret.text;
        if (ret.text.search("://") == -1) { // Disqualify things that don't come close to looking like URLs

            if (ret.text.match(/^file:\/[\w|\.]/)) {
                // unix file:/home.... url from drag/drop
                ret.text = 'file://'+ret.text.slice(5);
                log.debug('unix file url is now '+ret.text+'\n');
            } else {

                //dump('dropped data is '+ret.text+'\n');
                try {
                    var ioService = Components.classes["@mozilla.org/network/io-service;1"]
                                              .getService(Components.interfaces.nsIIOService);
                    var fileHandler = ioService.getProtocolHandler("file")
                                               .QueryInterface(Components.interfaces.nsIFileProtocolHandler);
                    ret.text = fileHandler.getURLSpecFromFile(origdata);
                    // fall through, magic presto it's now a URL
                } catch (e) {
                    log.debug('this is not a url: '+ret.text+'\n');
                    return;
                }

            }

        }
        log.debug('dropped URL is '+ret.text+'\n');
        ret.isURL = true;
        if (ret.text.search('kodebugger://') == 0) {
            ret.isDebuggerFile = true;
            return;
        } else
        if (ret.text.search('file://') == 0) {
            ret.isFileOrDir = true;
            ret.isFileURL = true;
            try {
                var path = uriparse_URIToLocalPath(uriparse_fixupURI(ret.text));
                ret.isExistingFile = os.path.exists(path);
                //dump("after exists test\n");
                if (ret.isExistingFile) {
                    ret.isDir = os.path.isdir(path);
                }
            } catch (e) {
                ret.isExistingFile = false;
            }
            return;
        } else
        if (ret.text.search('ftp://') == 0) {
            ret.isFileOrDir = true;
            ret.isFTPURL = true;
            res.isRemoteFileOrDir = true;
            // XXX should find out if it is a file on one of the servers we know about?
            // XXX should find out if it's a dir (hard?)
            return;
        } else
        if (ret.text.search('koremote://') == 0) {
            ret.isFileOrDir = true;
            ret.isRemoteFileOrDir = true;
            return;
        } else
        if (ret.text.search('https?://') == 0) {
            return;
        }
        // not a url after all
        ret.text = origdata;
        ret.isURL = false;
    break;
    }
}

