
function MacroManager() {
    this.recording = false;
    this._currentMacro = new Array();
    this._currentMacroText = '';
    this.log = getLoggingMgr().getLogger('MacroManager');
}


MacroManager.prototype = new Object();
MacroManager.prototype.constructor = MacroManager;

MacroManager.prototype.startRecording = function() {
    //dump("Starting to record\n");
    if (this._currentMacro) {
        delete this._currentMacro;
    }
    this._currentMacro = new Array();
    this._currentMacro.push("// Macro recorded on " + Date() + '\n');
    var macroVersionNumber = "2"; // must match the number in peMacro.js
    this._currentMacro.push('assertMacroVersion(' + macroVersionNumber + ');\n');
    this._currentMacro.push("gViewMgr.currentView.setFocus();\n");
    this._currentMacroText = new Array();
    this.recording = true;
    window.updateCommands("macro");
    StatusBar_AddMessage("Recording Macro", "macro", 0, true)
}

MacroManager.prototype.stopRecording = function() {
    this.recording = false;
    this._finishTextMacro();
    window.updateCommands("macro");
    StatusBar_AddMessage(null, "macro", 0, false)
    StatusBar_AddMessage("Macro Recorded", "macro", 3000, true)
}
MacroManager.prototype._finishTextMacro = function() {
    if (this._currentMacroText != '') {
        this._currentMacro.push("gViewMgr.currentView.selection = '" + 
                                this._currentMacroText.replace('\\', '\\\\', 'g').replace("\'", "\\'", 'g') +
                                "';\n");
        this._currentMacroText = '';
    }
}

MacroManager.prototype.appendCommand = function(command) {
    if (! this._currentMacro) {
        this.log.warn("Trying to append a command while not in a macro!!!");
        return;
    }
    this._finishTextMacro();
    this._currentMacro.push("command_doCommand('" + command +"');\n");
}

MacroManager.prototype.recordKeyPress = function(c) {
    if (! this._currentMacro) {
        this.log.warn("Trying to record keypresses while not in a macro!!!");
        return;
    }
    this._currentMacroText += c;
}

MacroManager.prototype.undo = function() {
    if (this._currentMacroText != '') {
        this._currentMacroText = this._currentMacroText.slice(0, this._currentMacroText.length-1);
    } else {
        this._currentMacro.pop();
    }
}

MacroManager.prototype.appendCode = function(code) {
    if (! this._currentMacro) {
        this.log.warn("Trying to append code while not in a macro!!!");
        return;
    }
    this._finishTextMacro();
    this._currentMacro.push(code);
}

MacroManager.prototype.executeMacro = function(macro) {
    var view = gViewMgr.currentView;
    if (view.getAttribute('type') != 'editor') {
        StatusBar_AddMessage("Can't execute macro as there is no focused editor", "macro", 3000, true)
        return;
    }
    scin = view.scimoz;

    scin.beginUndoAction();
    try {
        try {
            if (macro == null) {
                macro = this._currentMacro;
            }
            if (macro == null) return;
            if (this.recording) {
                this.recording = false; // avoid recursion!
            }
            // XXX weird stuff.
            // For some reason, Scintilla always thinks the lines are folded, which results in selection of
            // folded lines, which is typically not what we want So we unfold all the lines in the buffer.
            for (var lineno=0;lineno < scin.lineCount;lineno++)
                if ((scin.getFoldLevel(lineno) & scin.SC_FOLDLEVELHEADERFLAG) &&
                    !scin.getFoldExpanded(lineno))
                        scin.toggleFold(lineno)
            var code = (macro.join(''));
            MacroEvalCode(code);
        } catch (e) {
            this.log.error(e);
        }
    } finally {
        scin.endUndoAction();
    }
}

MacroManager.prototype.executeLastMacro = function(macro) {
    StatusBar_AddMessage("Executing Last Recorded Macro", "macro", 3000, false)
    this.executeMacro(this._currentMacro);
}

MacroManager.prototype.saveToToolbox = function(macro) {
    var os = Components.classes["@activestate.com/koOs;1"].getService();
    var koDirs = Components.classes["@activestate.com/koDirs;1"].
         getService(Components.interfaces.koIDirs);

    var macrosDir = os.path.join(koDirs.userDataDir, 'MyMacros');
    if (! os.path.exists(macrosDir)) {
        os.mkdir(macrosDir);
    }
    if (! os.path.isdir(macrosDir)) {
        this.log.error("Macros Directory " + macrosDir +
                          " does not exist or is not a directory!");
    }
    var fname = filepicker_saveFile(macrosDir, // default dir
                                    "MyMacro.js", // default filename
                                    "Save Macro As", // title 
                                    "JavaScript", // default filter name
                                    ["JavaScript", "All"]); // filters
    if (fname == null) {
        return;
    }
    
    os.writefile(fname, this._currentMacro.join(''));
    makeMacroFromFile(fname);
}

var gMacroMgr = new MacroManager();
