/* Copyright (c) 2000-2003 ActiveState Corporation.
   See the file LICENSE.txt for licensing information. */

// Note on "magic-pref-hookup-to-widget" handling:
//  The prefs magic-hookup-to-widget handling now allows one to specify that
//  multiple prefs are derived from one widget. This is done by specifying a
//  "prefstrings" attribute (instead of the usual "prefstring", singular) on a
//  widget where pref="true". The "prefstrings" attribute should be a
//  comma-separated list (NO SPACES!) of pref ids. Then, the identified pref
//  attribute should be a comma separated list of pref values. For example:
//
//    <menulist pref="true" prefattribute="data"
//              prefstrings="donotask_restore_workspace,donotask_action_restore_workspace">
//         <menupopup>
//             <menuitem data="false,"   label="Ask me what to do"/>
//             <menuitem data="true,Yes" label="Restore last workspace"/>
//             <menuitem data="true,No"  label="Open no files or projects"/>
//         </menupopup>
//    </menulist>
//
//  This functionality was originally added for the "Don't ask me again"
//  feature of the new dialogs in dialogs.js that use two prefs for their data.
//
//  NOTE: This feature disallows the use of commas in widget pref data (only
//  for the case of multiple-prefs-per-widget). If this ever becomes a problem
//  then we can start encoding this differently or using a different separator.
//

const _DEBUG = false;

function initPanel ( aPrefTag )
{
    hPrefWindow.onpageload( aPrefTag )
}

var prefLog = getLoggingMgr().getLogger('prefs');

function _getPrefAttributeForElement(elt) {
    var prefattr = elt.getAttribute("prefattribute");
    if (prefattr) // believe it :)
        return prefattr;
    // use a default
    switch (elt.tagName) {
        case "menulist":
        case "textbox":
            return "value";
        case "radio":
        case "radiogroup":
        case "checkbox":
            return "checked";
        default:
            prefLog.error("element '" + elt.getAttribute("id") + "' is of unknown type '" + elt.tagName + "' and has no 'prefattribute' tag");
    }
    return "value"; // I guess!
}

window.doneLoading = false; 
 
function koPrefWindow( frame_id, prefset /* = null */ )
{
    this.contentFrame   = frame_id
    this.prefset           = null;
    this.prefservice      = null;
    this.cancelHandlers = [];
    this.okHandlers     = [];
    this.customPageData = [];
    this.currentPageTag = null;
    this.helper = new prefTreeHelper("prefsTree");
    this.onload();
    this.init(prefset);
}

koPrefWindow.prototype =
{
    onload:	function () {
        // we really need a prefset per panel, but will always need a "global".
        try {
            this.prefservice = Components.classes["@activestate.com/koPrefService;1"].
                getService(Components.interfaces.koIPrefService);
        } catch(e) {
            dump("*** Failed to create preference service object\n");
            return;
        }
    },
    
    init: function (prefset /* = null */) {
        // setup the prefset.  If not specified, we use the global prefs.
        this.save_prefs_service = false;
        if (!prefset) {
            prefset = this.prefservice.prefs;
            this.save_prefs_service = true;
        }
        // Clone the prefset, so we can update that on the fly.
        // When OK is pressed, we update the original with our
        // modified preferences.
        this.prefset = prefset.clone();
        this.orig_prefset = prefset;

        // debug
        //cnt=new Object(); ids=new Object();this.prefset.getPrefIds(ids, cnt);
        //dump("Pref IDs using are " + ids.value + "\n");
    },

    _doOKHandlers: function() {
        for( var i = 0; i < this.okHandlers.length; i++ ) {
            var prefset = this._getCurrentPrefSet();
            var customData = this.customPageData[this.okHandlers[i].tag];
            if (!this.okHandlers[i].func(prefset, customData)) {
                prefLog.debug("OK handler for " + this.okHandlers[i].tag + " returned false - not closing dialog");
                return false;
            }
        }
        return true;
    },
    
    _doCancelHandlers: function() {
        for( var i = 0; i < this.cancelHandlers.length; i++ ) {
            var prefset = this._getCurrentPrefSet();
            var customData = this.customPageData[this.cancelHandlers[i].tag];
            if (!this.cancelHandlers[i].func(prefset, customData)) {
                prefLog.debug("Cancel handler for " + this.cancelHandlers[i].tag + " returned false - not closing dialog");
                return false;
            }
        }
        return true;
    },
    
    onOK: function () {
        // Save the prefs to our temp prefset, so the OK handlers see the new values.
        if (!this.savesinglepageprefs())
            return false;
        
        if (!this._doOKHandlers())
            return false;

        this.savePrefs();

        // persist which preference panel is currently open
        var selectedId = this.helper.getSelectedItemId();
        parent.opener.gPrefs.setStringPref('defaultSelectedPref', selectedId);

        this.setResult('ok');
        close();
        return true;
    },

    onCancel: function () {
        if (!this._doCancelHandlers())
            return false;
        
        // persist which preference panel is currently open
        var selectedId = this.helper.getSelectedItemId();
        parent.opener.gPrefs.setStringPref('defaultSelectedPref', selectedId);

        this.setResult('cancel');
        close();
        return true;
    },

    setResult: function (res) {
        if (window.arguments && window.arguments[1])
            window.arguments[1].res=res;
        return true;
    },	

    registerOKCallbackFunc: function ( aFunctionReference, aPrefTag ) {
        // check not already there
        if (aPrefTag) {
            for (var i=0;i<this.okHandlers.length;i++)
                if (this.okHandlers[i].tag == aPrefTag)
                    return;
        }
        this.okHandlers[this.okHandlers.length] = {
            func: aFunctionReference,
            tag: aPrefTag
        };
    },

    registerCancelCallbackFunc: function ( aFunctionReference, aPrefTag ) {
        // check not already there
        if (aPrefTag) {
            for (var i=0;i<this.cancelHandlers.length;i++)
                if (this.cancelHandlers[i].tag == aPrefTag)
                    return;
        }
        this.cancelHandlers[this.cancelHandlers.length] = {
            func: aFunctionReference,
            tag: aPrefTag
        };
    },

    // might end up with per-page prefsets.
    _getCurrentPrefSet: function() {
        return this.prefset;
    },

    // Get the DOM element that is the parent of the current pref widgets
    // (ie, the object we can enumerate to find the fields.
    _getPrefElementsParent: function() {
        var element_parent = document;
        if (this.contentFrame) {
            /***
            dump("content frame is '" + this.contentFrame + "'\n");
            dump("Frame[0] is " + window.frames[0] + "\n");
            dump("Frame[0].doc is " + window.frames[0].document + "\n");
            koDumpDOM(window.frames[0]);
            element_parent = window.frames[this.contentFrame].document;
            dump("got the element parent as " + element_parent + "\n");
            koDumpDOM(element_parent);
            ***/
            element_parent = window.frames[0].document; // XXXX - WTF is the right thing here!?!?!?!
        }
        return element_parent;
    },

    _setElementFromPrefValue: function(elt) {
        // Set a UI element from its preference value(s).
        // Returns the preference value (which may not be the label on the widget,
        // especially for menu lists, radios, etc
        var prefIds = null;
        if (elt.hasAttribute("prefstring")) {
            prefIds = [elt.getAttribute("prefstring")];
        } else if (elt.hasAttribute("prefstrings")) {
            prefIds = elt.getAttribute("prefstrings").split(",");
        } else {
            prefIds = [elt.id];
        }
        var prefValues = [];
        for (i = 0; i < prefIds.length; ++i) {
            prefValues.push(this.getPref(prefIds[i]));
        }

        var prefattribute, items, item, i, widgetValues;
        switch (elt.tagName) {
        case "menulist":
            prefattribute = _getPrefAttributeForElement(elt);
            var popup = elt.getElementsByTagName("menupopup")[0];

            items = elt.getElementsByTagName("menuitem");
            //dump("menulist has " + items.length + " items, prefValues are " +
            //     prefValues.join(",") + "\n");
            for (i = 0; i < items.length; i++) {
                item = items[i];
                if (prefIds.length > 1) {
                    widgetValues = item.getAttribute(prefattribute).split(",");
                    if (widgetValues.length != prefIds.length) {
                        var err = "Incorrect number of values in pref data for <"+elt.tagName+"> element associated with pref ids '"+prefIds.join(',')+"'. The number of pref data values must match the number of pref ids: value='"+item.getAttribute(prefattribute)+"'";
                        prefLog.error(err)
                        alert("Internal Error: "+err);
                        continue;
                    }
                } else /* prefIds.length == 1 */ {
                    widgetValues = [item.getAttribute(prefattribute)];
                }
                if (widgetValues.join(',') == prefValues.join(',')) { // just want to compare lists
                    elt.selectedItem = item;
                    elt.setAttribute("value", prefValues.join(',')); //XXX Why this?
                    return;
                }
            }
            break;

        case "radiogroup":
            prefattribute = _getPrefAttributeForElement(elt);
            items = elt.getElementsByTagName("radio");
            for (i = 0; i < items.length; i++) {
                item = items[i];
                if (prefIds.length > 1) {
                    widgetValues = item.getAttribute(prefattribute).split(",");
                } else /* prefIds.length == 1 */ {
                    widgetValues = [item.getAttribute(prefattribute)];
                }
                if (widgetValues.join(',') == prefValues.join(',')) { // just want to compare lists
                    //dump("pref values are " + prefValues.join(',') +
                    //     " Item value is " + item.getAttribute(prefattribute) +
                    //     ", label is " + item.getAttribute("label") + "\n");
                    item.checked = true;
                    elt.selectedItem = item;
                    return;
                }
            }

        default:
            if (elt.getAttribute("prefwidget") == "true") {
                elt.value = prefValues.join(',');
                return;
            } else {
                prefattribute = _getPrefAttributeForElement(elt);
                // XX - is this needed?
                // seems a shame that textboxes are different?
                //dump("Set prefs '" + prefIds.join(',') + "' to values '" +
                //     prefValues.join(',') + "'\n");
                if (elt.tagName == "textbox" && prefattribute == "value") {
                    elt.value = prefValues.join(',');
                } else {
                    elt.setAttribute(prefattribute, prefValues.join(','));
                }
                return;
            }
        }
        prefLog.error("Could not set widget '" + elt.id + "' (pref ids '" +
                      prefIds.join(',') + "') to values '" +
                      prefValues.join(',') + "'");
        throw("Invalid preference");
        return;
    },

    _getPrefValueFromElement: function(elt) {
        // Given a UI widget, get the value for the preference.
        //dump("_getPrefValue for ID " + elt.tagName + "\n");
        var prefattribute, items, i;
        switch (elt.tagName) {
        case "menulist":
            prefattribute = _getPrefAttributeForElement(elt);
            var popup = elt.getElementsByTagName("menupopup")[0];
            var popup_value = elt.getAttribute("label");
            
            items = elt.getElementsByTagName("menuitem");
            for (i=0;i<items.length;i++) {
                var item = items[i];
                if (item.getAttribute("label") == popup_value) {
                    //dump("preference value for popup is " + item.getAttribute("value") + "\n");
                    return item.getAttribute(prefattribute);
                }
            }
            break;

        case "radiogroup":
            prefattribute = _getPrefAttributeForElement(elt);
            items = elt.getElementsByTagName("radio");
            if (elt.selectedItem) {
                //dump("radiogroup has " + items.length + " items, selected item is "+elt.selectedItem.getAttribute(prefattribute)+"\n");
                return elt.selectedItem.getAttribute(prefattribute)
            }
            break;

        default:
            if (elt.getAttribute( "prefwidget" )=="true") {
                return elt.value;
            } else {
                var prefattr = _getPrefAttributeForElement(elt);
                var value;

                //XXX is the special handling of textboxes really needed?
                if (elt.tagName == "textbox" && prefattr == "value")
                    value = elt.value;
                else
                    value = elt.getAttribute( prefattr );
                //dump("widget '" + elt.id + "set preference val to '" + value + "'\n");
                return value;
            }
        }
        prefLog.error("Could not get preference value from element '" + elt.id + "'");
        throw("Invalid preference");
        return null; // shut-up strict mode.
    },
    
    hasPrefHere: function (aPrefString) {
        var prefs = this._getCurrentPrefSet();
        return prefs.hasPrefHere(aPrefString);
    },

    getPref: function (aPrefString) {
        var prefs = this._getCurrentPrefSet();		
        var aPrefType = prefs.getPrefType(aPrefString);
        if (aPrefType == null) {
            var msg = "pref.id is UNDEFINED, aPrefString = " + aPrefString;
            prefLog.error(msg);
            throw msg;
        }
        switch ( aPrefType ) {
        case "boolean":
            return prefs.getBooleanPref(aPrefString);
            break;
        case "long":
            return prefs.getLongPref(aPrefString);
            break;
        case "double":
            return prefs.getDoublePref(aPrefString);
            break;
        case "string":
            return prefs.getStringPref(aPrefString);
            break;
        default:
            return prefs.getPref(aPrefString);
            break;
        }
        return null;  // shutup strict mode
    },
    
    setPref: function ( prefId, value ) {
        var prefs = this._getCurrentPrefSet();
        var prefType = prefs.getPrefType(prefId);
        if (prefType == "boolean") {
            if (typeof(value)=="string")
                value = value == "true" ? true : false;
            prefs.setBooleanPref(prefId, value);
        } else if (prefType == "long") {
            prefs.setLongPref(prefId, value);
        } else if (prefType == "double") {
            prefs.setDoublePref(prefId, value);
        } else if (prefType == "string") {
            prefs.setStringPref(prefId, value);
        } else {
            throw "Unknown preference type ("+prefType+")";
        }
    },

    savePrefs: function () {
        this.orig_prefset.update(this.prefset);
        if (this.save_prefs_service) {
            this.prefservice.saveState();
        }
    },

    _doPageUnload: function() {
        if (!this.savesinglepageprefs())
            return false;
        var prefset = this._getCurrentPrefSet();
        var pageTag = this.currentPageTag;

        // Now the preference set is up to date, call any custom page unloaders to update custom data.
        if( pageTag && this.contentFrame && ('OnPreferencePageUnloading' in window.frames[ this.contentFrame ])) { // is there a start function.
            var custom_data = this.customPageData[pageTag];
            window.frames[ this.contentFrame ].OnPreferencePageUnloading(prefset, custom_data);
        }
        return true;
    },
    switchPage: function () {
        var selectedRow = this.helper.getSelectedRow();
        var oldURL = document.getElementById( this.contentFrame ).getAttribute("src");
        var newURL = selectedRow.firstChild.getAttribute("url");

        if (newURL == oldURL)
            return;

        if (oldURL)
            this._doPageUnload();
        document.getElementById(this.contentFrame).setAttribute("src", newURL);
        // Page OnLoad scripts fire, which end up calling initPage()
        // (which should die - may as well just auto-trigger.)
    },

    // return the widget on the current page with the pref ID, or null.
    findElementWithPreferenceId: function(prefId) {
        var elements = this._getPrefElementsParent().getElementsByAttribute("pref", "true");
        for (var i = 0; i < elements.length; i++) {
            var element = elements[i];
            var widgetPrefId = element.hasAttribute("prefstring") ? element.getAttribute("prefstring") : element.id;
            if (prefId == widgetPrefId)
                return element;
        }
        return null;
    },

    onSinglePageOK: function () {
        if (!this._doPageUnload())
            return false;
        if (!this._doOKHandlers())
            return false;
        this.savePrefs();
        return true;
    },

    savesinglepageprefs: function () {
        var i, j;
        var elements = this._getPrefElementsParent().getElementsByAttribute( "pref", "true" );
        var rc = true;
        for (i = 0; i < elements.length; i++) {
            var element = elements[i];
            var prefIds = null;
            if (element.hasAttribute("prefstring")) {
                prefIds = [element.getAttribute("prefstring")];
            } else if (element.hasAttribute("prefstrings")) {
                prefIds = element.getAttribute("prefstrings").split(",");
            } else {
                prefIds = [element.id];
            }

            try {
                var widgetValues;
                if (prefIds.length > 1) {
                    widgetValues = this._getPrefValueFromElement(element).split(',');
                    if (widgetValues.length != prefIds.length) {
                        var err = "Incorrect number of values in pref data for <"+element.tagName+"> element associated with pref ids '"+prefIds.join(',')+"'. The number of pref data values must match the number of pref id: value='"+this._getPrefValueFromElement(element)+"'";
                        prefLog.error(err)
                        alert("Internal Error: "+err);
                        continue;
                    }
                } else /* prefIds.length == 1 */ {
                    widgetValues = [this._getPrefValueFromElement(element)];
                }

                for (j = 0; j < prefIds.length; j++) {
                    var prefValue = this.getPref(prefIds[j]);
                    if (this.hasPrefHere(prefIds[j]) &&
                        (prefValue == widgetValues[j] ||
                         (widgetValues[j] == "true"  && prefValue === true) ||
                         (widgetValues[j] == "false" && prefValue === false))){
                        // do nothing, the value has not changed
                    } else {
                        //dump("savesinglepageprefs: setting preference ID '" +
                        //     prefIds[j] + "' to value '" +
                        //     widgetValues[j] +"'\n");
                        this.setPref(prefIds[j], widgetValues[j]);
                    }
                }
            } catch (e) {
                prefLog.error("Could not save prefs '" + prefIds.join(',') +
                              "' to values '" + widgetValues.join(',') + "': " +
                              e + "\n");
                rc = false; // but keep updating what prefs we can.
            }
        }
        // look for pref-widgets and save the prefs
        var prefWidgets = this._getPrefElementsParent().getElementsByAttribute( "prefwidget", "true" );
        for( i = 0; i < prefWidgets.length; i++ ) {
            var widget = prefWidgets[i];
            //dump("calling saveprefs for "+widget.getAttribute("prefstring")+"\n");
            widget.saveprefs(this);
        }
        return rc;
    },

    onsinglepageload: function () {
        this.onpageload(null);
    },

    onpageload: function ( aPageTag ) {
        this.currentPageTag = aPageTag;
        var prefset = this._getCurrentPrefSet();
        // First see if we need the once-only "OnPreferencePageInitalize" function.
        if ( aPageTag && !(aPageTag in this.customPageData) ) { // no custom page data - first time we have seen it.
            var new_custom_data = null;
            if( this.contentFrame && ('OnPreferencePageInitalize' in window.frames[ this.contentFrame ])) { // is there a start function.
                try {
                    new_custom_data = window.frames[ this.contentFrame ].OnPreferencePageInitalize(prefset);
                } catch (e) {
                    prefLog.error("Calling 'OnPreferencePageInitalize' function for page '" + aPageTag + "' + failed: " + e);
                }
            }
            this.customPageData[aPageTag] = new_custom_data;
        }
        
        // look for pref-widgets and initialize them
        var prefWidgets = this._getPrefElementsParent().getElementsByAttribute( "prefwidget", "true" );
        for( var i = 0; i < prefWidgets.length; i++ ) {
            var widget = prefWidgets[i];
            //dump("calling onpageload for "+widget.getAttribute("prefstring")+"\n");
            widget.onpageload(this);
        }

        // Setup all our widget values.
        var prefElements = this._getPrefElementsParent().getElementsByAttribute( "pref", "true" );
        //dump("page load loading " + prefElements.length + " items\n");
        for( i = 0; i < prefElements.length; i++ ) {
            var element = prefElements[i];
            this._setElementFromPrefValue(element);
        }
        // Call the page loading function.
        if( aPageTag && this.contentFrame && ('OnPreferencePageLoading' in window.frames[ this.contentFrame ])) { // is there a start function.
            var custom_data = this.customPageData[aPageTag];
            window.frames[ this.contentFrame ].OnPreferencePageLoading(prefset, custom_data);
        }
    },

    closeBranches: function ( aComponentName ) {
        var panelChildren = document.getElementById( "panelChildren" );
        var panelTree = document.getElementById( "prefsTree" );
        for( var i = 0; i < panelChildren.childNodes.length; i++ ) {
            var currentItem = panelChildren.childNodes[i];
            if( currentItem.id != aComponentName && currentItem.id != "appearance" )
                currentItem.removeAttribute( "open" );
        }
        var openItem = document.getElementById( aComponentName );
        panelTree.selectItem( openItem );
    }
};



/* tree support functions
 
   these functions can be used if trees are coded in xul and you are
   not using a view class like the one above.  Primarily this helps
   in retreiving the selected treerow.
   
   XXX This certainly could be optimized
   but this is just a quick fix for moving to 1.3
*/
function prefTreeHelper(treeid) {
    this.tree = document.getElementById(treeid);
}
prefTreeHelper.prototype = {
    getTreeEntries: function(treechildren, all_entries) {
        var entries = [];
        for (var i=0;i<treechildren.childNodes.length;i++) {
            var treeitem = treechildren.childNodes[i];
            entries[entries.length] = treeitem.firstChild; // treerow
            if (treeitem.getAttribute('container') == 'true' &&
                (all_entries || treeitem.getAttribute('open') == 'true')) {
                var children = this.getTreeEntries(treeitem.firstChild.nextSibling,all_entries);
                if (children.length > 0)
                    entries = entries.concat(children);
            }
        }
        return entries;
    },
    getVisibleRowByIndex: function(index) {
        this.entries = this.getTreeEntries(this.tree.firstChild.nextSibling,false);
        return this.entries[index];
    },
    getVisibleRowById: function(id) {
        this.entries = this.getTreeEntries(this.tree.firstChild.nextSibling,
                                           false);
        for (var i=0; i < this.entries.length; i++) {
            if (this.entries[i].parentNode.id == id)
                return i - 1;
        }
        return -1;
    },
    getRowIndexById: function(id) {
        this.entries = this.getTreeEntries(this.tree.firstChild.nextSibling,true);
        for (var i=0; i < this.entries.length; i++) {
            if (this.entries[i].parentNode.id == id) return i - 1;
        }
        return -1;
    },
    ensureRowVisible: function(row) {
        if (row.parentNode.parentNode.tagName == 'treeitem' &&
            row.parentNode.parentNode.getAttribute('open') != 'true')
        {
            row.parentNode.parentNode.setAttribute('open', 'true');
            //XXX I am not sure that this is correct. I would think the arg
            //    should be "row.parentNode.parentNode". However, we currently
            //    have not prefs panels more than 2 levels deep so it
            //    doesn't really matter.
            this.ensureRowVisible(row.parentNode.parentNode.firstChild);
        }
    },
    selectRowById: function(id) {
        var row = document.getElementById(id);
        if (!row) return;
        this.ensureRowVisible(row);
        var index = this.getVisibleRowById(id);
        this.tree.view.selection.select(index+1);
    },
    /* returns the current selection (treerow element). */
    getSelectedRow: function() {
        return this.getVisibleRowByIndex(this.tree.currentIndex);
    },
    /* returns the id of the treeitem currently selected */
    getSelectedItemId: function() {
        return this.getVisibleRowByIndex(this.tree.currentIndex).parentNode.getAttribute('id');
    },
    /* returns an array of selected items (treerow elements). */
    getSelectedRows: function() {
        var entries = this.getTreeEntries(this.tree.firstChild.nextSibling,false);
        var start = new Object();
        var end = new Object();
        var numRanges = this.tree.view.selection.getRangeCount();
        var selection = [];
        for (var t=0; t<numRanges; t++){
          tree.view.selection.getRangeAt(t,start,end);
          for (var v=start.value; v<=end.value; v++){
            selection[selection.length] = entries[v];
          }
        }
        return selection;
    }
}

