/* Copyright (c) 2000-2003 ActiveState Corporation.
   See the file LICENSE.txt for licensing information. */

// Constants and global variables
const tag = 'chrome://jaguar/content/pref/pref-keys.xul';
var dialog  = {}; 

var _elementIDs = ["keybinding-scheme"];
	
// Handlers

function PrefKeys_OnLoad() {
    try {
        dialog.tree = document.getElementById('keybinding-tree');
        dialog.configurations= document.getElementById('configurations');
        dialog.configpopup= document.getElementById('configpopup');
        dialog.gKeybindingMgr = parent.opener.gKeybindingMgr;
        dialog.editkeybinding = document.getElementById('keybinding');
        dialog.editkeybinding.gKeybindingMgr = dialog.gKeybindingMgr;
        dialog.mainwindow = dialog.editkeybinding.mainwindow = parent.opener;
        dialog.gKeybindingMgr.parseGlobalData();
        dialog.deleteButton = document.getElementById('deleteConfiguration');
        updateUI();
        updateConfigMenu();
        updateCommandList()
        parent.hPrefWindow.onpageload(tag);
        parent.hPrefWindow.registerOKCallbackFunc(PrefKeys_OkCallback, tag);
        parent.hPrefWindow.registerCancelCallbackFunc(PrefKeys_CancelCallback, tag);
    } catch (e) {
        parent.prefLog.error(e)
    }
}

function OnPreferencePageInitalize(prefset) 
{
    var data = new Object();
    dialog.prefset = prefset;
    data.keybindingMgr = dialog.gKeybindingMgr;
    dialog.originalConfiguration = dialog.gKeybindingMgr.currentConfiguration;
    return data;
}

// Start up & returning to this panel in a session
function OnPreferencePageLoading(prefset, data) 
{
    dialog.prefset = prefset;    
}

// Switches to another prefs panel
function OnPreferencePageUnloading(prefset, data) 
{
}

function PrefKeys_CancelCallback(prefset, data) {
    try {
        if (typeof(dialog) != 'undefined' && dialog && dialog.gKeybindingMgr) {
            dialog.gKeybindingMgr.revertToPref(dialog.originalConfiguration);
        }
    } catch (e) {
        parent.prefLog.error("Error in PrefKeys_CancelCallback" + e);
    }
    return true;
}

function PrefKeys_OkCallback(prefset, data) {
    try {
        if (typeof(dialog) != 'undefined' && dialog && dialog.gKeybindingMgr) {
            dialog.gKeybindingMgr.saveAndApply(prefset);
        }
    } catch (e) {
        parent.prefLog.error("Error in PrefKeys_OkCallback" + e);
    }
    return true;
}

// Callbacks and global functions having to do with configuration handling

function updateConfigMenu() {
    // Update the configuration menu based on the list of known configurations

    var configs = dialog.gKeybindingMgr.getConfigurations();
    var currentConfig = dialog.gKeybindingMgr.currentConfiguration;
    var menuitem;
    while (dialog.configpopup.hasChildNodes()) {
        dialog.configpopup.removeChild(dialog.configpopup.lastChild);
    }
    var c;
    var selectedmenuitem = null;
    for (i in configs) {
        menuitem = document.createElement('menuitem');
        c = configs[i];
        menuitem.setAttribute('label', c);
        menuitem.setAttribute('id', c);
        if (! dialog.gKeybindingMgr.configurationWriteable(c)) {
            menuitem.setAttribute('class','primary_menu_item');
        }
        menuitem.setAttribute('oncommand', "return switchConfig(\""+configs[i]+"\");");
        dialog.configpopup.appendChild(menuitem);
        if (c == currentConfig) {
            selectedmenuitem = menuitem;
        }
    }
    if (selectedmenuitem) {
        dialog.configurations.selectedItem = selectedmenuitem;
    } else {
        dialog.configurations.selectedItem = dialog.configpopup.firstChild;
    }
    dialog.configurations.setAttribute("label", dialog.gKeybindingMgr.currentConfiguration);
}


function switchConfig(configname) {
    // called when users switch to another configuration from the menulist
    dialog.gKeybindingMgr.switchConfiguration(configname, dialog.prefset);
    updateUI();
}

function updateUI() {
    try {
        var configname = dialog.gKeybindingMgr.currentConfiguration;
        dialog.configurations.selectedItem = document.getElementById(configname)
        if (dialog.gKeybindingMgr.configurationWriteable(configname)) {
            dialog.deleteButton.removeAttribute('disabled');
        } else {
            dialog.deleteButton.setAttribute('disabled', 'true');
        }
        dialog.editkeybinding.updateCurrentKey();
    } catch (e) {
        log.exception(e);
    }
}

function deleteConfiguration()  {
    // called by the Delete button next to the configuration menu list.
    var configname = dialog.configurations.selectedItem.getAttribute('id');
    if (typeof(configname) == 'undefined') {
        parent.prefLog.error("SelectedItem doesn't have a data attribute");
        alert("There was a problem delete the configuration.");
        return;
    }
    var shortname = configname;
    if (dialog_yesNo("Are you sure you want to delete the scheme '" + shortname +"'?  This action cannot be undone.") == 'No') {
        return;
    }
    dialog.gKeybindingMgr.deleteConfiguration(configname, dialog.prefset);
    updateConfigMenu();
    updateUI();
}

function selectKey(event, treeitem)  {
    // Called by the handler in the command list tree.
    var item = null;
    if (typeof(treeitem) != 'undefined' && treeitem)
        item = treeitem;
    else {
        item = dialog.tree.selectedItem;
    }
    if (item) {
        dialog.editkeybinding.commandId = item.getAttribute('name');
        dialog.editkeybinding.updateCurrentKey();
    }
}

function updateCommandList() {
    // Add treeitems to the tree based on the information from the keybinding manager.
    // This is slow, which is why it's inlined.
    var i, command; 
    var commanditems = dialog.gKeybindingMgr.commanditems
    for (i in commanditems) {
        command = commanditems[i];
        var item = dialog.tree.appendItem(command.desc,command);
        item.setAttribute('name', command.name);
        item.setAttribute('data', command.desc);
    }
}

function newConfiguration()
{
    var currentConfig = dialog.gKeybindingMgr.currentConfiguration;
    var shortname = currentConfig;
    var newSchemeName;
    var i, ok;
    var fnameRe = /^[\w\d ]*$/;
    var knownConfigs = dialog.gKeybindingMgr.getConfigurations();
    while (1) {
        newSchemeName = dialog_prompt("Enter a new scheme name.  The new scheme will be based on the currently selected '" +
                                      shortname + "' scheme.", "New Scheme Name:",
                                      newSchemeName // default value
                                      );
        if (!newSchemeName) {
            return false;
        }
        ok = true;
        // Check to make sure that the name isn't already taken and that it can be written to disk.
        for (i = 0; i < knownConfigs.length; i++) {
            if (knownConfigs[i] == newSchemeName) {
                alert("The scheme name '" + newSchemeName + "' is already used.  Please choose another.")
                ok = false;
                break;
            }
        }
        if (!ok) continue;
        if (fnameRe.test(newSchemeName)) {
            ok = true;
            break;
        }
        alert("Scheme names must contain alphanumeric characters, spaces or underscores only.  Please choose another name.");
    }
    newSchemeName = dialog.gKeybindingMgr.makeNewConfiguration(newSchemeName, dialog.prefset);
    dialog.gKeybindingMgr.switchConfiguration(newSchemeName, dialog.prefset);
    updateConfigMenu();
    updateUI();
    return true;
}

