/* Copyright (c) 2000-2001 ActiveState Corporation.
   See the file LICENSE.txt for licensing information. */

// Globals
function projectTreeView(treeWidget, manager) {
  partTreeView.apply(this, [treeWidget, manager]); // inherit from partTreeView, defined in partTreeView.js
  this.datapoints = {};
}

// The following two lines ensure proper inheritance (see Flanagan, p. 144).
projectTreeView.prototype = new partTreeView();
projectTreeView.prototype.constructor = projectTreeView;

projectTreeView.prototype.getSelectedProjectName = function() {
    var selection = this.entriesList[this.mSelection.currentIndex];
    if (selection.type == 'project') {
        return selection.name;
    }
    return null;
}

projectTreeView.prototype.roundDigits =  function(n,d) {
    n = Math.round(n * 100) / 100;
    n = (n + 0.001) + '';
    return n.substring(0, n.indexOf('.') + d);
}

projectTreeView.prototype.getCellText = function(row, colID) {
    var res = "";
    if (row < this.entriesList.length) {
        var item = this.entriesList[row];
        if (typeof(item) == 'undefined') return res;
        //dump('get cell text for '+colID+'\n');
        switch (colID) {
        case 'Name':
            if (item.type == 'extensionfolder') {
                extension = item.part.getStringAttribute('extension')
                if (extension == '') {
                    res = "Files with no extension";
                } else {
                    res = extension+' files'
                }
            } else if (item.type =='languagefolder') {
                res = item.part.getStringAttribute('language') +' files'
            } else if (item.type == 'project' && item.part.isDirty) {
                res = item.name+"*";
            } else {
                res = item.name;
            }
            break;
        case 'Size':
            if (typeof(item.properties['Size']) != 'undefined') {
                res =item.properties['Size'];
            } else if (typeof(item.properties['fileSize']) != 'undefined' &&
                       item.properties['fileSize']) {
                var size = parseInt(item.properties['fileSize']);
                if (size < 1024) {
                    size = size + ' bytes';
                } else if (size < 1024*1024) {
                    size = Math.ceil(size / 1024);
                    size = size + ' K';
                } else {
                    size = this.roundDigits(size / (1024 * 1024),2);
                    size = size + 'MB';
                }
                item.properties['Size'] = res = size;
            }
            break;
        case 'Date':
            if (typeof(item.properties['Date']) != 'undefined') {
                res = item.properties['Date'];
            } else if (typeof(item.properties['lastModifiedTime']) != 'undefined' &&
                       item.properties['lastModifiedTime']) {
                var date = new Date(parseInt(item.properties['lastModifiedTime'])*1000);
                res = item.properties['Date'] = getDateTimeString(date);
            }
            break;
        default:
            var fieldname = this.datapoints[colID];
            if (typeof(item.properties[fieldname]) != 'undefined') {
                res = item.properties[fieldname];
            }
            break;
        }
    }
    //dump("projectTreeView.getCellText("+row+", "+colID+")=["+res+"]\n");
    return res;
}

var gProjectImages = {
    'default': {
        'default': 'chrome://jaguar/skin/images/file_icon.png',
        'command': 'chrome://jaguar/skin/images/run_commands.png',
        'snippet': 'chrome://jaguar/skin/images/snippet.gif',
        'URL': 'chrome://jaguar/skin/images/xlink.gif',
        'template': 'chrome://jaguar/skin/images/newTemplate.png',
        'DirectoryShortcut': 'chrome://jaguar/skin/images/open.png',
        'WebService': 'chrome://jaguar/skin/images/webServicesImg.png',
        'macro': 'chrome://jaguar/skin/images/macro.png',
        'missing': 'chrome://jaguar/skin/images/file_icon_missing.png',
        'projectFolderOpen': 'chrome://jaguar/skin/images/folder-open.png',
        'projectFolderClosed': 'chrome://jaguar/skin/images/folder-closed.png',
        'projectVirtualFolderOpen': 'chrome://jaguar/skin/images/folder-open.png',
        'projectVirtualFolderClosed': 'chrome://jaguar/skin/images/folder-closed.png'
    },
    
    'project' : {
        'default': {
            'default':'chrome://jaguar/skin/images/project_icon.png',
            'missing': 'chrome://jaguar/skin/images/file_icon_missing.png',
            'sccOk': 'chrome://jaguar/skin/images/project_icon_ok.png',
            'sccSync': 'chrome://jaguar/skin/images/project_icon_sync.png',
            'sccConflict': 'chrome://jaguar/skin/images/project_icon_conflict.png',
            'add': 'chrome://jaguar/skin/images/project_icon_add.png',
            'delete': 'chrome://jaguar/skin/images/project_icon_delete.png'
        },
        'isReadOnly': {
            'default': 'chrome://jaguar/skin/images/project_icon_readonly.png',
            'sccOk': 'chrome://jaguar/skin/images/project_icon_readonly_ok.png',
            'sccSync': 'chrome://jaguar/skin/images/project_icon_readonly_sync.png',
            'sccConflict': 'chrome://jaguar/skin/images/project_icon_readonly_conflict.png'
        },
        'edit': {
            'default': 'chrome://jaguar/skin/images/project_icon_edit.png',
            'sccOk': 'chrome://jaguar/skin/images/project_icon_edit_ok.png',
            'sccSync': 'chrome://jaguar/skin/images/project_icon_edit_sync.png',
            'sccConflict': 'chrome://jaguar/skin/images/project_icon_edit_conflict.png'
        }
    },
    
    'projectFile' : {
        'default': {
            'default': 'chrome://jaguar/skin/images/file_icon.png',
            'missing': 'chrome://jaguar/skin/images/file_icon_missing.png',
            'sccOk': "chrome://jaguar/skin/images/file_icon_ok.png",
            'sccSync': "chrome://jaguar/skin/images/file_icon_sync.png",
            'sccConflict': "chrome://jaguar/skin/images/file_icon_conflict.png",
            'add': "chrome://jaguar/skin/images/file_icon_add.png",
            'delete': "chrome://jaguar/skin/images/file_icon_delete.png"
        },
        'isReadOnly': {
            'default':'chrome://jaguar/skin/images/file_icon_readonly.png',
            'sccOk':'chrome://jaguar/skin/images/file_icon_readonly_ok.png',
            'sccSync':'chrome://jaguar/skin/images/file_icon_readonly_sync.png',
            'sccConflict':'chrome://jaguar/skin/images/file_icon_readonly_conflict.png',
            'add': "chrome://jaguar/skin/images/file_icon_readonly_add.png",
            'delete': "chrome://jaguar/skin/images/file_icon_readonly_delete.png"
        },
        'edit': {
            'default': "chrome://jaguar/skin/images/file_icon_edit.png",
            'sccOk': "chrome://jaguar/skin/images/file_icon_edit_ok.png",
            'sccSync': "chrome://jaguar/skin/images/file_icon_edit_sync.png",
            'sccConflict': "chrome://jaguar/skin/images/file_icon_edit_conflict.png"
        }
    },

    'guifile' : {
        'default': {
            'default': 'chrome://jaguar/skin/images/GUIB.gif',
            'missing': 'chrome://jaguar/skin/images/GUIB.gif',
            'sccOk': "chrome://jaguar/skin/images/GUIB_ok.gif",
            'sccSync': "chrome://jaguar/skin/images/GUIB_sync.gif",
            'sccConflict': "chrome://jaguar/skin/images/GUIB_conflict.gif",
            'add': "chrome://jaguar/skin/images/GUIB_add.gif",
            'delete': "chrome://jaguar/skin/images/GUIB_delete.gif"
        },
        'isReadOnly': {
            'default':'chrome://jaguar/skin/images/GUIB_readonly.gif',
            'sccOk':'chrome://jaguar/skin/images/GUIB_readonly_ok.gif',
            'sccSync':'chrome://jaguar/skin/images/GUIB_readonly_sync.gif',
            'sccConflict':'chrome://jaguar/skin/images/GUIB_readonly_conflict.gif'
        },
        'edit': {
            'default': "chrome://jaguar/skin/images/GUIB_edit.gif",
            'sccOk': "chrome://jaguar/skin/images/GUIB_edit_ok.gif",
            'sccSync': "chrome://jaguar/skin/images/GUIB_edit_sync.gif",
            'sccConflict': "chrome://jaguar/skin/images/GUIB_edit_conflict.gif"
        }
    },













    '':{} /* js strict, no trailing comma's allowed */
};

projectTreeView.prototype._getImageSrcPath = function(properties, images) {
    for (var i=0; i< properties.length; i++) {
        if (typeof(images[properties[i]]) != 'undefined') {
            var image = images[properties[i]];
            if (typeof(image) == 'string') return image;
            return this._getImageSrcPath(properties, image);
        }
    }
    if (typeof(images['default']) != 'string')
        return this._getImageSrcPath(properties,images['default']);
    return images['default'];
}

  /* AString getImageSrc(in long row, in wstring colID); */
projectTreeView.prototype.getImageSrc = function(row, colID) {
    if (row >= this.entriesList.length || typeof(this.entriesList[row]) == 'undefined') return null;
    if (colID != 'Name') return null;
    var item = this.entriesList[row];
    var properties=[];
    
    // get the properties for the icon
    if (item.isContainer && item.isContainerOpen) {
        for (i in item.iconopenstyle) {
            if (item.iconopenstyle[i])
                properties[properties.length]=i;
        }
    } else {
        for (i in item.iconstyle) {
            if (item.iconstyle[i])
                properties[properties.length]=i;
                
        }
    }
    // see if we have an icon matching these properties
    if (properties.length > 0) {
        var path = this._getImageSrcPath(properties,gProjectImages);
        if (path) {
            return path;
        }
    }
    // no icon, use default icons
    if (item.isContainer) {
        // generic icons for files and folders if no style exists for the parts icon
        if (item.isContainerOpen) {
            return gProjectImages['default']['projectVirtualFolderOpen'];
        } else {
            return gProjectImages['default']['projectVirtualFolderClosed'];
        }
    }
    return gProjectImages['default']['default'];
}

projectTreeView.prototype._setProperties = function(stylearray, properties) {
    var done = false;
    for (var style in stylearray) {
        if (stylearray[style]) {
            //dump('setting style '+style+'\n');
            properties.AppendElement(this.atomService.getAtom(style));
            done=true;
        }
    }
    return done;
}

  /* void getCellProperties(in long row, in wstring colID, in nsISupportsArray properties); */
projectTreeView.prototype.getCellProperties = function(row, colID, properties) {
    // plugins set the properties
    if (row >= this.entriesList.length || typeof(this.entriesList[row]) == 'undefined') return;
    var item = this.entriesList[row];
    if (item.isContainerOpen)
        this._setProperties(item.iconopenstyle, properties);
    else
        this._setProperties(item.iconstyle, properties);
}

projectTreeView.prototype.updateSortIndicators = function(sortId)
{
    var fieldname = null;
    for (var id in this.datapoints) {
        if (sortId == this.datapoints[id]) {
            fieldname = id;
            break;
        }
    }
    if (!fieldname) return;
    
    var currCol = this.manager.viewMgr.tree.firstChild.firstChild;
    var sortedColumn = currCol;
    // set the sort indicator on the column we are sorted by
    if (sortId) {
        while (sortedColumn) {
            if (sortedColumn.localName == "treecol" &&
               sortedColumn.getAttribute('id') == fieldname) break;
            sortedColumn = sortedColumn.nextSibling;
        }
        if (sortedColumn && sortedColumn.getAttribute('id') == fieldname) {
            var sortDirection = sortedColumn.getAttribute("sortDirection");
            if (sortDirection && sortDirection == "ascending") {
                sortedColumn.setAttribute("sortDirection", "descending");
            } else {
                sortedColumn.setAttribute("sortDirection", "ascending");
            }
        }
    }

    // remove the sort indicator from all the columns
    // except the one we are sorted by
    while (currCol) {
        while (currCol && currCol.localName != "treecol") {
            currCol = currCol.nextSibling;
        }
        if (currCol && (currCol != sortedColumn)) {
            currCol.removeAttribute("sortDirection");
        }
        if (currCol) {
            currCol = currCol.nextSibling;
        }
    }
}


var projectTooltipShowTimer = null;
var projectTooltipCloseTimer = null;
function cancelProjectTooltipTimer()
{
    if (projectTooltipShowTimer) window.clearTimeout(projectTooltipShowTimer);
    projectTooltipShowTimer = null;
    if (projectTooltipCloseTimer) window.clearTimeout(projectTooltipCloseTimer);
    projectTooltipCloseTimer = null;
}

function setProjectTooltipTimer(tooltipName, elName, x,y)
{
    if (projectTooltipCloseTimer) window.clearTimeout(projectTooltipCloseTimer);
    dump('showProjectTooltip("'+tooltipName+'", "'+elName+'",'+x+', '+y+');\n');
    projectTooltipShowTimer = window.setTimeout('showProjectTooltip("'+tooltipName+'", "'+elName+'",'+x+', '+y+');', 1000);
}

function closeProjectTooltip(tooltipName)
{
    cancelProjectTooltipTimer();
    document.getElementById(tooltipName).hidePopup();
}

function showProjectTooltip(tooltipName, elName, x,y)
{
    document.getElementById(tooltipName).hidePopup();
    var el = document.getElementById(elName).tooltipBaseElement;
    document.getElementById(tooltipName).showPopup(el, x, y, 'tooltip');
    projectTooltipCloseTimer =
        window.setTimeout('closeProjectTooltip("'+tooltipName+'");', 5000);
}
