# Copyright (c) 2000-2001 ActiveState Corporation.
# See the file LICENSE.txt for licensing information.

# The Perl debug engine
from xpcom import components, _xpcom, nsError, ServerException, COMException
#from xpcom.server.enumerator import SimpleEnumerator
import bdb
import thread, threading
import os, time
import traceback, sys

import dbgutils
import re, string
import socket
import cPickle as pickle
import urllib
import uriparse
import logging

RESUME_STOP = components.interfaces.koIDebugger.RESUME_STOP
RESUME_STEP_IN = components.interfaces.koIDebugger.RESUME_STEP_IN
RESUME_STEP_OVER = components.interfaces.koIDebugger.RESUME_STEP_OVER
RESUME_STEP_OUT = components.interfaces.koIDebugger.RESUME_STEP_OUT
RESUME_GO = components.interfaces.koIDebugger.RESUME_GO

DBG_STATE_STOPPED = components.interfaces.koIDebugger.DBG_STATE_STOPPED
DBG_STATE_BREAK = components.interfaces.koIDebugger.DBG_STATE_BREAK
DBG_STATE_RUNNING = components.interfaces.koIDebugger.DBG_STATE_RUNNING

log = logging.getLogger("koPerlDebugger")

def _canonicalizeFileName(fullName):
    matchObject = re.match(r'.*[/\\]([^/\\]*)', fullName)
    if matchObject:
        return matchObject.group(1)
    else:
        return fullName

def _getArrayFromString(sepEncodedString, simpleProtocolChar,complexProtocolChar):
    if not sepEncodedString:
        return []
    elif sepEncodedString[0] == simpleProtocolChar:
        # ses[0] == simpleProtocolChar
        # ses[1] == sepChar on rest of string
        actualArray = string.split(sepEncodedString[2:],
                                   sepEncodedString[1])
    elif sepEncodedString[0] == complexProtocolChar:
        # ses[0] == complexProtocolChar
        # ses[1] == sepChar on rest of string
        # ses[2] == escapeChar on rest of string
        # ses[3] == encodedSepChar on rest of string
        actualArray = string.split(sepEncodedString[4:],
                                   sepEncodedString[1])
        for i in range(len(actualArray)):
            # s/<esc><innocuous-char>/<sep>/g
            actualArray[i] = re.sub("%s%s" % (sepEncodedString[2],
                                              sepEncodedString[3]),
                                    sepEncodedString[1],
                                    actualArray[i])
            # s/<esc><esc>/<esc>/g
            actualArray[i] = re.sub("%s%s" % (sepEncodedString[2],
                                              sepEncodedString[2]),
                                    sepEncodedString[2],
                                    actualArray[i])
    else:
        log.error("koPerlDebugger._getArrayFromString: expecting a string starting with 'a' or 'A', got %s" % (sepEncodedString,))
        return []
    return actualArray

class koPerlBreakpoint(dbgutils.koBreakpointBase):
    def __init__(self, engine, where, line):
        dbgutils.koBreakpointBase.__init__(self, engine, where, line)
        self.added_to_debugger = 0
    def get_language(self):
        return "Perl"

    def _updatePending(self):
        if not self.pending:
            return

        pd = self.engine.perl_debugger
        cf = _canonicalizeFileName(pd.DoCommand("GetFileName"))
        cf2 = _canonicalizeFileName(self.filename)
        if cf2 != cf:
            # Tell Perl which filename to use
            breakLocation = "%s:%d" % (cf2, self.lineno)
        else:
            breakLocation = "%d" % (self.lineno, )
        self.pending = 0
        if self.state == components.interfaces.koIBreakpoint.BREAKPOINT_DELETED:
            if self.added_to_debugger:
                pd.DoCommand ("RemoveBreakpoint %s" % (breakLocation,) )
                self.added_to_debugger = 0
                # print "QQQ: Post RemoteBP: List Breakpoints:", pd.DoCommand("ListBreakPoints")
                # print "QQQ: Done Listing Breakpoints"
            return
        elif  self.state == components.interfaces.koIBreakpoint.BREAKPOINT_DISABLED:
            return
        # Either addition or modification - ensure we have a number
        if not self.added_to_debugger:
            # print "QQQ: Breakpoints: about to set bp at ", breakLocation
            resp = pd.DoCommand("InsertBreakpoint %s" % (breakLocation,) )
            # print "QQQ: Perl InsertBreakpoint returned", repr(resp)
            # we get strings back as response
            if resp and resp != '0':
                self.added_to_debugger = 1
                # Dump the breakpoints...
                # print "QQQ: Post InsertBP: List Breakpoints:", pd.DoCommand("ListBreakPoints")
                # print "QQQ: Done Listing Breakpoints"
            else:
                #failure, set back to pending
                self.added_to_debugger = 0
                self.pending = 1
                # figure out if the breakpoint is valid or not
                le = pd.DoCommand("GetLastResult")
                if le and le.lower().find('not breakable') >= 0:
                    self.state = components.interfaces.koIBreakpoint.BREAKPOINT_DISABLED
                

        # OK - valid breakpoint with some state -set it.
        #### enabled = (self.state == components.interfaces.koIBreakpoint.BREAKPOINT_ENABLED)
        # And do what with it!?  Does Perl's debugger have the concept of "disabled"?
        # No, all breakpoints in Perl are always enabled.

def varSort(a,b):
    # use lower case and remove .'s
    return cmp(a.name[1:].lower(), b.name[1:].lower())

def arrayVarSort(a,b):
    # remove brackets and do a numeric sort
    if a.name.startswith('->'):
        a = a.name[3:-1]
    else:
        a = a.name[1:-1]
    if b.name.startswith('->'):
        b = b.name[3:-1]
    else:
        b = b.name[1:-1]
    return cmp(int(a), int(b))

def hashVarSort(a,b):
    # use lower case and remove braces
    return cmp(a.name[1:-1].lower(), b.name[1:-1].lower())


# For now, just a variable.
class koPerlDebugProperty:
    _com_interfaces_ = components.interfaces.koIDebugProperty
    def __init__(self, dbg, name, fullName, radix, stackFrame):
        self.dbg = dbg
        self.name = name
        self.fullName = fullName
        self.calculatedValue = 0
        # Defer evaluation until we need it.
        self.value = None
        self.stackFrame = stackFrame

    def get_name( self ):
        # Result: wstring
        return self.name
    
    def get_fullName( self ):
        # Result: wstring
        # Maybe add the context?
        return self.fullName
    
    def get_type( self ):
        # Result: wstring
        return self.name[0]

    def get_value( self ):
        # Result: wstring
        if not self.calculatedValue:
            if self.fullName[0] == '@':
                val = "ARRAY"
            elif self.fullName[0] == '%':
                val = "HASH"
            elif self.stackFrame._cache.has_key(self.fullName):
                val = self.stackFrame._cache[self.fullName]
            else:
                #print "QQQ: not cached: [%s]" % (self.fullName,)
                # Allow for glitches that trigger syntax errors due to
                # crud in the pipe.
                for i in range(2):
                    val = self.dbg.DoCommand("EvalExpression overload::StrVal(%s)" % (self.fullName,))
                    #print "This look for '%s' got '%s'" % (self.fullName, val)
                    if re.match(r'syntax error at \(eval\s+\d+\)', val):
                        continue
                    else:
                        break
            self.value = val
            self.calculatedValue = 1
        #print "QQQ: koPerlDebugger.py -- value of %s => %s" % (self.fullName, self.value)
        # note: binary data kills komodo, repr prevents that.
        try:
            result = unicode(self.value)
        except:
            # if we cannot convert to unicode, then simply treat the
            # data as if it is binary.  This will provide \xXX strings
            result = repr(self.value)[1:-1] # ignore quotes
        return result
    
    def get_hasChildren( self ):
        if self.name[0] in ['@', '%']: return 1;
        else:
            if self.stackFrame._refCache.has_key(self.fullName):
                # print "QQQ: yes ref-cached: [%s] => [%s]" % (self.fullName, self.stackFrame._refCache[self.fullName])
                refType = self.stackFrame._refCache.get(self.fullName)
            else:
                # print "QQQ: not ref-cached: [%s]" % (self.fullName,)
                refType = self.dbg.DoCommand("EvalExpression ref(%s)" % (self.fullName,))
            if refType is None or len(refType) == 0:
                val = 0
            elif refType in ['CODE','GLOB','LVALUE']:
                val = 0
            elif refType in ['SCALAR','ARRAY','HASH','REF']:
                val = 1
            elif re.match(r'[a-zA-Z_][a-zA-Z0-9_]*::[a-zA-Z0-9_:]+$', refType):
                val = 1
            elif re.match(r'[a-zA-Z_][a-zA-Z0-9_]*$', refType):
                log.warn("Accepting %s as a Perl reference type." % (refType,))
                val = 1
            else:
                val = 0
            if not val:
                # print "QQQ: ref(%s) => '%s', has no children" % (self.fullName, refType)
                pass
            else:
                # print "QQQ: ref(%s) => '%s', does have children" % (self.fullName, refType)
                pass
            return val
        # Result: boolean
    
    def SetValueAsString( self, value ):
        # Result: void - None
        # In: param0: wstring
        # Out: err_msg: wstring
        if self.get_hasChildren():
            # Don't allow compound objects to be assigned
            return "Compound objects can not have their value changed."
            raise ServerException(nsError.NS_ERROR_UNEXPECTED)
        elif re.match('[_a-zA-Z][a-zA-Z0-9_:]*$', value):
            pass
        elif value[0] == '\'':
            pass
        elif value[0] == '"':
            pass
        else:
            # Assume it's a string that's fully escaped.
            #todo: escape the string into Perl syntax
            value = '"' + value + '"'
        _str = self.dbg.DoCommand("RelayToDebugger %s = %s" %
                                 (self.fullName, value))
        #print "koPerlDebugger.py:QQQ: attempted var assignment returned '%s'" % (_str,)
        if _str:
            return _str
        return None
    
    def enumChildren( self ):
        vars=[]
        varType = None
        isRef = 0
        # print "QQQ: calc children of", self.fullName

        # Figure out what kind of array we're looking at.
        
        if self.fullName[0] == '@':
            varType = "a"
            isRef = 0
            refFullName = "\\%s" % (self.fullName,)
        elif self.name[0] == '%':
            varType = "h"
            isRef = 0
            refFullName = "\\%s" % (self.fullName,)
        else:
            refFullName = self.fullName
            val = self.dbg.DoCommand("EvalExpression overload::StrVal(%s)" % (self.fullName,))
            if re.search(r'ARRAY\(0x[0-9a-f]+\)', val):
                varType = "a"
                isRef = 1
            elif re.search(r'HASH\(0x[0-9a-f]+\)', val):
                varType = "h"
                isRef = 1
            else:
                # Treat it like a plain reference: dereference, and work
                # on it in the next round.
                # Nothing more to do here.
                # print "QQQ: Punting on eval of %s" % (self.fullName, )
                vars.append(koPerlDebugProperty(self.dbg, '$',
                                                "${%s}" % (self.fullName, ),
                                                "radix??",
                                                self.stackFrame))
                #return SimpleEnumerator(vars)
                return vars
        
        # Get the child-refs in one shot, and cache them.
        # We have to get the refs before we get the values.

        #todo: encapsulate this in the listener / jacket
        # koPerlDebugger shouldn't know about DB::ASRemote.

        if varType == 'a':
            sepEncodedString = self.dbg.DoCommand("EvalExpression DB::ASRemote::escapeArrayChildrenRefs(%s)" % (refFullName,))
            actualArray = _getArrayFromString(sepEncodedString, 'a', 'A')
            for i in range(len(actualArray)):
                if isRef: newFullName = "%s->[%d]" % (self.fullName, i)
                else: newFullName = "$%s[%d]" % (self.fullName[1:] ,i)
                self.stackFrame._refCache[newFullName] = actualArray[i]
                # print "QQQ: refCached %s[%s]" % (newFullName, actualArray[i])
                
        elif varType == 'h':
            sepEncodedString = self.dbg.DoCommand("EvalExpression DB::ASRemote::escapeHashChildrenRefs(%s)" % (refFullName,))
            actualArray = _getArrayFromString(sepEncodedString, 'h', 'H')
            for i in range(len(actualArray) / 2):
                thisKey = actualArray[i * 2]
                thisVal = actualArray[i * 2 + 1]
                
                if isRef: newFullName = "%s->{'%s'}" % (self.fullName, thisKey)
                else: newFullName = "$%s{'%s'}" % (self.fullName[1:], thisKey)
                self.stackFrame._refCache[newFullName] = thisVal
                # print "QQQ: refCached %s[%s]" % (newFullName, thisVal)
        else:
            log.error("Expected var type of a or h, got" + str(varType))
            #return SimpleEnumerator([])
            return []

        if isRef:
            evalName = self.fullName
        else:
            evalName = "\\" + self.fullName

        if varType == 'a':
            sepEncodedString = self.dbg.DoCommand("EvalExpression DB::ASRemote::escapeArrayChildren(%s)" % (evalName,))
            actualArray = _getArrayFromString(sepEncodedString, 'a', 'A')
            for i in range(len(actualArray)):
                if isRef:
                    newFullName = "%s->[%d]" % (self.fullName, i)
                    newPartName = "->[%d]" % (i,)
                else:
                    newFullName = "$%s[%d]" % (self.fullName[1:], i)
                    newPartName = "[%d]" % (i,)
                self.stackFrame._cache[newFullName] = actualArray[i]
                # print "QQQ: cached [%s]" % (newFullName,)
                vars.append(koPerlDebugProperty(self.dbg, newPartName,
                                                newFullName,
                                                "radix??", self.stackFrame))
        elif varType == 'h':
            sepEncodedString = self.dbg.DoCommand("EvalExpression DB::ASRemote::escapeHashChildren(%s)" % (evalName,))
            actualArray = _getArrayFromString(sepEncodedString, 'h', 'H')
            for i in range(len(actualArray) / 2):
                thisKey = actualArray[i * 2]
                thisVal = actualArray[i * 2 + 1]

                quotableKey = thisKey.replace("\\", "\\\\").replace("'", "\\'")
                if isRef:
                    newFullName = "%s->{'%s'}" % (self.fullName, quotableKey)
                    newPartName = "->{'%s'}" % (thisKey,)
                else:
                    newFullName = "$%s{'%s'}" % (self.fullName[1:], quotableKey)
                    newPartName = "{'%s'}" % (thisKey,)
                self.stackFrame._cache[newFullName] = thisVal
                # print "QQQ: cached %s" % (newFullName,)
                vars.append(koPerlDebugProperty(self.dbg, "{%s}" % (thisKey,),
                                                newFullName,
                                                "radix??",
                                                self.stackFrame))
                
        if varType == 'a':
            vars.sort(arrayVarSort)
        elif varType == 'h':
            vars.sort(hashVarSort)
        else:
            vars.sort(varSort)
        # Result: nsISimpleEnumerator
        #return SimpleEnumerator(vars)
        # Result is array now!
        return vars

class koPerlDebugStackFrame:
    _com_interfaces_ = components.interfaces.koIDebugStackFrame

    def __init__(self, engine, dbg, baseURI, filename, lineno, constructUri):
        self.engine = engine
        self.dbg = dbg
        self.filename = os.path.normpath(filename)
        self.lineno = lineno
        self.constructUri = constructUri
        self._cache = {}
        self._refCache = {}
        self.uri = None
        # baseURI is not a URI, it's something that derives
        # a new URI "based" on a common base.
        self.baseURI = baseURI.QueryInterface(
                           components.interfaces.koIDebuggerURI)

        # perl doesnt use these, but we need to init them
        self.inputBaseURI = ""
        self.inputURI = ""
        self.inputLineno = 0
        self.inputWhere = ""

    def _get_basic_info(self):
        rel_uri = self.filename.replace("\\", "/")
        # print "QQQ:2 _get_basic_info: rel_uri is", rel_uri
        try:
            resourceMapper = components.classes[
                "@activestate.com/resource-mapper;1"]\
                .getService(components.interfaces.koIResourceMapper)
            
            uri = resourceMapper.mapURL(self.baseURI.newURI(rel_uri).spec, None)
            # print "QQQ:3 _get_basic_info: self.uri is", self.uri
        except:
            traceback.print_exc(sys.stdout)
            if rel_uri[1] == ':':
                uri = "file:///" + rel_uri
            else:
                uri = "file://" + rel_uri
        self.uri = self.constructUri(uri)

    def get_uri(self):
        try:
            if self.uri is None:
                self._get_basic_info()
        except:
            traceback.print_exc(sys.stdout)

        return self.uri

    def get_filename( self ):
        # Result: string
        #todo: XXX Use koPerlDebugStackFrame.uri rather than .filename
        return self.filename

    def get_lineNumber( self ):
        # Result: uint32
        return self.lineno
    def get_description( self ):
        # Result: wstring
        return "%s, line %d" % (self.filename, self.lineno)
    def get_language( self ):
        # Result: wstring
        return "Perl"

    def getContextNames(self):
        # print "QQQ: called getContextNames"
        return ["Proximity", "Registers"]

    def prefetchValues(self, expressions):
        """Retrieve the values of the expressions and their references
        from the listener and stash them in the cache for fast access"""
        # XXX UGLY HACK
        # see bug http://bugs.activestate.com/show_bug.cgi?id=22513
        # for some reason, a request for a large number of variables
        # makes perl5db barf.  splitting it up into smaller chunks tends
        # to let things ride.
        while expressions:
            batch, expressions = expressions[:20], expressions[20:]
            # get the variable values and cache them for efficient access
            command = "EvalExpressions " + " ".join(batch)
            pickledData = self.dbg.DoCommand(command)
            if not pickledData: return
            try:
                results = pickle.loads(pickledData)
            except:
                # If this happens and we fail to cache any values,
                # the variable viewer will have to query for values one
                # at a time (and thus be slower), but will still work.
                if not sys.version.startswith("2.0"):
                    log.error("Failed to unpickle variable data: the pickle"
                              "format may have changed since Python 2.0");
                else:
                    log.error("Failed to unpickle variable data")
            else:
                # results is a flat tuple of value, reference pairs
                for i in range(0, 2*len(batch), 2):
                    self._cache[batch[i/2]] = results[i]
                    self._refCache[batch[i/2]] = results[i+1]

    def enumContext(self, context):
        # Result: nsISimpleEnumerator
        # In: context: string

        # We're in the UI thread, so use this one.
        perl_debugger = self.dbg
        if context == "Proximity":  # more like proximity vars.
            # Grab our source 5 lines back and 4 lines down, and
            # search for variable names.

            proxBack = 4
            proxForward = 5
            textToAnalyze = self.engine.getSourceCode(self.filename)
            if len(textToAnalyze) == 0:
                # There isn't anything yet.
                # print "QQQ: no text returned by SourceFile"
                #return None
                return []
            textArray=string.split(textToAnalyze, "\n")
            curLine = self.lineno - 1
            startBack = curLine
            while startBack >= 0:
                if re.match(r'sub\b', textArray[startBack]):
                    break
                startBack -= 1
                    
            if startBack < 0:
                startBack = 0

            if proxBack > curLine: proxBack = curLine

            if startBack > curLine - proxBack:
                startBack = curLine - proxBack

            # Now show the text to analyze
            fwdLine = curLine + proxForward
            maxLine = len(textArray)
            if fwdLine > maxLine:
                fwdLine = maxLine

            textToAnalyze = string.join(textArray[startBack:fwdLine], "\n")
            # Assume a user-defined Perl var has at least one lower-char alpha char.
            vars1 = re.findall(r'[@%][a-zA-Z_][a-zA-Z0-9_]*(?:::[a-zA-Z_][a-zA-Z0-9_]*)*\{?', textToAnalyze)
            vars2 = re.findall(r'\$[a-zA-Z_][a-zA-Z0-9_]*(?:::[a-zA-Z_][a-zA-Z0-9_]*)*[\{\[]?', textToAnalyze)
            # Convert hash slices back into hashes
            for i in range(len(vars1)):
                if vars1[i][-1] == '{':
                    vars1[i] = "%%%s" % vars1[i][1:-1]
                
            # Convert indexed globals back to the other thing
            for i in range(len(vars2)):
                if vars2[i][-1] == '{':
                    vars2[i] = "%%%s" % vars2[i][1:-1]
                elif vars2[i][-1] == '[':
                    vars2[i] = "@%s" % vars2[i][1:-1]
            vars = []
            varnames = []
            self._cache = {}
            # print "QQQ: Clearing the cache"
            self._refCache = {}
            # Filter out duplicates, and sort result ignoring leading %...
            for var in vars1 + vars2:
                if not var in varnames:
                    varnames.append(var)
            if not varnames:
                #return SimpleEnumerator([])
                return []
            
            self.prefetchValues(varnames)
            for var in varnames:
                if self._cache.has_key(var) and self._cache[var] is not None:
                    vars.append(koPerlDebugProperty(perl_debugger, var, var, "radix??", self))
            vars.sort(varSort)
            # print "QQQ: we think the vars are", [v.name for v in vars]
            #return SimpleEnumerator(vars)
            return vars
    
        elif context == "Registers":  # more like register vars.
            regNameStr="_.&`'123456789"
            varnames = []
            vars = []
            for c in regNameStr:
                name = "$%s" % c
                varnames.append(name)
            self.prefetchValues(varnames)
            for var in varnames:
                if self._cache[var] is not None:
                    vars.append(koPerlDebugProperty(perl_debugger, var,
                                                    var, "radix??", self))
            vars.sort(varSort)
            # print "QQQ: we think the vars are", [v.name for v in vars]
            #return SimpleEnumerator(vars)
            return vars
        
        #return SimpleEnumerator([])
        return []
    
    def getExpression(self, expr, radix):
        return koPerlDebugProperty(self.dbg, expr, expr, radix, self)

class RedirectedIOHandler:
    def __init__(self, engine, port, stdin, stdout, stderr):
        self.port = port
        self.engine = engine
        self.stdin = stdin
        self.stdout = stdout
        self.stderr = stderr
        self.consoleSemaphore = threading.Semaphore(0)

    def start_listening(self):
        threading.Thread(target = self._listener_thread).start()

    def _listener_thread(self):
        # Get output from the process, and relay it to either
        # stdout or stderr.
        #
        # XXX todo: support stdin as well
        #if __debug__: print "koPerlDebugger._create_process_listener(): setting up socket, listening on port %d" % (IOPort,)
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.bind(("", self.port)) # listen on localhost
        # try to re-use a server port if possible
        s.setsockopt(
            socket.SOL_SOCKET, socket.SO_REUSEADDR,
            s.getsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR) | 1)
        s.listen(1)
        # print "About to release the semaphore."
        self.consoleSemaphore.release()
        #if __debug__: print "koPerlDebugger._create_process_listener(): listening"
        (client, addr) = s.accept()
        # client is a NEW socket
        s.close()
        #if __debug__: print "koPerlDebugger._create_process_listener(): connection made at attr <client=%s,addr=%s>" % (`client`, `addr`)
        while 1:
            try:
                data = client.recv(4098)
                if len(data) == 0:
                    # log.debug("QQQ: read 0 bytes from socket.")
                    break
            except:
                #log.warn("QQQ: failed to read data from socket.")
                break
            self.stdout.write(data)
        client.close()
        # Tell the main loop we have finished incase that main loop
        # did not initiate the termination
        try:
            self.engine.resume(RESUME_STOP)
        except COMException:
            pass # no problemo

        self.stderr.write('\n' + '=' * 10 + "Perl process finished. " + '=' * 10 + "\n")
        # drop any cycles.  "engine" is most important.
        self.stdout = self.stderr = self.stdin = self.engine = None

    def stop_listening(self):
        # print "stop_listening is about to acquire the semaphore, even though I am not quite sure what it is here for!"
        self.consoleSemaphore.acquire()
        try:
            s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            s.connect(('127.0.0.1', self.port))
            s.close()
        except socket.error:
            log.error("Socket error trying to stop the I/O listener on port %d" % (IOPort,))
            pass

class koPerlDebugEngine(dbgutils.koDebugEngineBase):
    _com_interfaces_ = components.interfaces.koIDebugEngine
    _reg_clsid_ = "{87bb9680-975e-4d4f-8e7c-06bed1d1403e}"
    _reg_contractid_ = "@activestate.com/debugger_engine_Perl;1"
    
    baseURI = None
    
    def __init__(self):
        dbgutils.koDebugEngineBase.__init__(self)

        sm = _xpcom.GetGlobalServiceManager()
        self.komodo_debugger = components.classes["@activestate.com/debugger;1"] \
                              .getService(components.interfaces.koIDebuggingFramework)
        self.perl_debugger = None
        self.process_spawner = None
        self.language = "Perl"
        self._is_remote = 0
        # log.debug("QQQ: Perl debugger engine initialized ok.")

    def _check_debug_listener(self):
        if self.perl_debugger is None:
            self._init_debug_listener()
            assert self.perl_debugger is not None, "Failed to create the listener, but didnt raise an exception!"

    def _init_debug_listener(self):
        self.perl_debugger = components.classes["@activestate.com/RemoteDebugConnectionListener;1"] \
			   .getService(components.interfaces.koIDebugConnectionListener).perlListener;

    def startDebugging(self, stepType, preferences, stdin, stdout, stderr):
        
        self._check_debug_listener()
        self._refreshBreakpoints()
        self._clearFileCache()

        handler = components.classes[
                        "@mozilla.org/network/protocol;1?name=kodebugger"].\
                  getService(components.interfaces.koIDebuggerProtocolHandler)
        handler.registerEngine(self,"Perl")

        resourceConverter = components.classes[
            "@activestate.com/koDebuggerToFileResourceConverter;1"] \
            .createInstance(components.interfaces.koIResourceConverter)
        resourceMapper = components.classes[
            "@activestate.com/resource-mapper;1"]\
            .getService(components.interfaces.koIResourceMapper)

        resourceMapper.registerMapping(self.baseURI.spec,
                                       "file:///",
                                       resourceConverter)

        # The cwd preference is not necessarily defined for local
        # debugging, and neither cwd nor filename will be defined for
        # the remote case.

        if preferences.hasStringPref("cwd") and preferences.getStringPref("cwd"):
            self.cwd = preferences.getStringPref("cwd")
        elif preferences.hasStringPref("filename"):
            script = preferences.getStringPref("filename")
            self.cwd = os.path.dirname(script)
        else:
            self.cwd = None
        
        if stdin is not None or stdout is not None or stderr is not None:
            # create a redirection port.
            # This is a hack - not sure what the solution is though!
            self._is_remote = 1
            if not preferences.hasLongPref("debugger.io-port"):
                preferences.setLongPref("debugger.io-port", preferences.getLongPref("debugger.listener-reponse.port") + 1)
            IOPort = preferences.getLongPref("debugger.io-port")
            self.io_redirector = RedirectedIOHandler(self, IOPort, stdin, stdout, stderr)
            self.io_redirector.start_listening()
            exclude_protocols = ""
        else:
            self._is_remote = 0
            self.io_redirector = None
            exclude_protocols = "file"
            
        self.uriConstructor = lambda originalUrl, newProtocol = self.baseURI.spec , exceptForProtocols = [exclude_protocols]: \
                                constructURL( originalUrl, newProtocol, exceptForProtocols )


        threading.Thread(target = self._do_debug,
                         args=(stepType,preferences)).start()
        return 1

    def _do_debug(self, step_type, preferences):
        # When a debugger contacts the listener, we'll get the
        # source over the wire, and debug it.
        #cmdPort = preferences.getLongPref("debugger.listener-reponse.port")
        IOPort = preferences.getLongPref("debugger.io-port")
        try:
            # This method initializes the Perl debugger jacket, and one
            # of the first things it does is start listening for
            # connections on port 2000.
            #
#            print "QQQ: Before calling self.perl_debugger.Init() on port", cmdPort
            # 3rd "param" to init is called "show console".  It really means "dont_attempt_io_redirection"
            #initialize the debugger now
            res = self.perl_debugger.DoCommand ("InitDebugger " +
                                          str(IOPort) + ':' +
                                          str(self.io_redirector is None) + ":" +
                                          str(self._is_remote))
            
            # print "QQQ: After calling self.perl_debugger.Init(), res = %d" % (res,)
            if res == 0:
                # Close the I/O listener
                if self.io_redirector:
                    self.io_redirector.stop_listening()
                    self.io_redirector = None
                # print 'QQQ: About to call Term() just in case'
                self.perl_debugger.Term()
                return

            try: # finally - terminate the debugger
                # Send any breakpoints we know about before we really get started.
                self._processPendingBreakpoints()
                # If we only want to stop at the first b/p, do that now.
                # Do the "GetFileName" to ensure we are
                rc = 1
                fname = self.perl_debugger.DoCommand ("GetFileName")
                if step_type == RESUME_GO and fname:
                    lineNo = self.perl_debugger.DoCommand ("GetLineNumber")
                    # This is needed to get around the problem that
                    # the perl debugger doesn't have a "go" command, it just
                    # has a 'start' command. 
                    if not self._haveBreakpoint(fname, lineNo):
                        self._set_state(DBG_STATE_RUNNING, 0)
                        rc  = self.perl_debugger.DoCommand("Continue")

                pat = re.compile(r'\(eval\s+\d+\)\s*\[')
                while rc != '0':
                    # Check if we're still debugging the program.
                    lnum = self.perl_debugger.DoCommand("GetLineNumber")
                    fname = self.perl_debugger.DoCommand ("GetFileName")
                    # print "_do_debug:QQQ: filename=%s, line=%s" % (fname, lnum)
                    # If the program isn't at a well-defined location, assume it's
                    # finished.
                    if lnum == "" or fname == "":
                        # print "QQQ: leaving loop: lnum='%s', fname = '%s'" % (lnum, fname)
                        break
                    elif not re.search(pat, fname):
                        # if we are in a running state, that means user
                        # did F5-RUN, and is not STEPing.  If we also
                        # have a pending breakpoint in that file, this
                        # means we likely did a break on load in
                        # a required file.  This is about as close as we
                        # can aproximate this issue without rewriting
                        # more functionality
                        if self.state == DBG_STATE_RUNNING and \
                           self._havePendingBreakpoint(fname):
                            self._processPendingBreakpoints()
                            resume = RESUME_GO
                        else:
                            resume = self._runCommandLoop(components.interfaces.koIDebugger.BREAK_REASON_BREAK)
                    else:
                        # print "QQQ: not calling _runCommandLoop -- lnum='%s', fname = '%s'" % (lnum, fname)
                        pass
                        
                    if resume == RESUME_STOP:
                        break
                    elif resume == RESUME_STEP_IN:
                        rc = self.perl_debugger.DoCommand("StepInto")
                    elif resume == RESUME_STEP_OVER:
                        rc = self.perl_debugger.DoCommand("StepOver")
                    elif resume == RESUME_STEP_OUT:
                        rc = self.perl_debugger.DoCommand("StepOut")
                    elif resume == RESUME_GO:
                        rc = self.perl_debugger.DoCommand("Continue")
                    else:
                        log.error("****** Unknown resume code: %s - ignored" % (resume,))
            finally:
                self.perl_debugger.Term()
        finally:
            # log.debug("Perl debugger terminating.")
            self._set_state(DBG_STATE_STOPPED, components.interfaces.koIDebugger.STOP_REASON_STOPPED)
            self.perl_debugger = None

    def _haveBreakpoint(self, fname, lineNo):
        cf = _canonicalizeFileName(fname)
        try:
            self.bplock.acquire()
            for bkpt in self.breakpoints:
                cf2 = _canonicalizeFileName(bkpt.filename)
                if cf == cf2 and int(bkpt.lineno) == int(lineNo):
                    return 1
            return 0
        finally:
            self.bplock.release()

    def _havePendingBreakpoint(self, fname):
        cf = _canonicalizeFileName(fname)
        try:
            self.bplock.acquire()
            for bkpt in self.breakpoints:
                if not bkpt.pending: continue
                cf2 = _canonicalizeFileName(bkpt.filename)
                if cf == cf2:
                    return 1
            return 0
        finally:
            self.bplock.release()

    def _enumStackFrames( self ):
        # print "QQQ: _enumStackFrames"
        # Result: nsISimpleEnumerator
        if self.stack_frame_list is None:
            # XXX = hack - pretend there is only ever one frame
            # as I don't know how to walk.
            cs = self.perl_debugger.DoCommand ("CallStack")
            fileName = self.perl_debugger.DoCommand ("GetFileName")
            if (self._is_remote \
                or fileName[0] == '/' \
                or fileName[0] == '\\' \
                or (re.match(r'\w', fileName[0]) and fileName[1] == ':')):
                lineNumber = self.perl_debugger.DoCommand("GetLineNumber")
            else:
                matchObject = re.search(r'\(eval\s+\d+\)\s*\[(.*):(\d+)\]',
                                        fileName)
                if (matchObject):
                    fileName = matchObject.group(1)
                    lineNumber = matchObject.group(2)
                else:
                    lineNumber = self.perl_debugger.DoCommand("GetLineNumber")
                if self.cwd:
                    fileName = os.path.join(self.cwd, fileName)
            try:
                lineNumber = int(lineNumber)
            except ValueError:
                # print "*** Perl gave a bogus line number -", `lineNumber`
                lineNumber = 1

            files = [fileName]
            lines = [lineNumber]
            if cs and cs != 0:
                callStack1 = string.split(cs, "\n")
                for cs1 in callStack1:
                    matchObject = re.search(r'called from file `(.*)\' line (\d+)', cs1)
                    if matchObject:
                        files.append(matchObject.group(1))
                        lines.append(int(matchObject.group(2)))
                    else:
                        pass
                    
            # then build the frames.
            frames = []
            for i in range(len(files)):
                frames.append(koPerlDebugStackFrame(self,
                                                    self.perl_debugger,
                                                    self.baseURI,
                                                    files[i], lines[i], self.uriConstructor))
            self.stack_frame_list = frames
            
        else:
            pass
        #return SimpleEnumerator(self.stack_frame_list)
        return self.stack_frame_list

    def _clearFileCache( self ):
        """clear any cached files.  this should be called
        before initiating a new debugging session."""

        self._cachedFileName = None
        self._cachedFileContents = None

    def _getSourceCode( self, fileName ):
        """we cache the current file to avoid repeated transfers.
        only one for now, to avoid bloat."""

        #XXX we assume this will only be called with the Perl debugger's
        #XXX current filename.  this is always true in current usage
        #XXX but may not be, in general.  the listener and jacket should
        #XXX be expanded to handle non-current files.

        if fileName != self._cachedFileName:
            self._cachedFileName = fileName
            self._cachedFileContents = self.perl_debugger.DoCommand("SourceFile")
        return self._cachedFileContents

    def breakNow( self ):
        # Result: void - None
        pass

    def _stop( self ):
        # Result: void - None
        pass

    def addBreakpoint( self, where, lineNumber ):
        # Result: koIBreakpoint
        # print "QQQ:koPerlDebugger.py:addBreakpoint: %s:%d" % (where, lineNumber)
        ret = koPerlBreakpoint(self, where, lineNumber)
        # We must add the BP to the list before updating it!
        self.breakpoints.append(ret)
        ret._tryUpdatePending()
        return ret
        
    def getContextNames(self):
        return ["vars"]

def constructURL( originalUrl, prefix, exceptForProtocols = [] ):
    path = uriparse.URIToLocalPath(originalUrl)
    if path:
        path = os.path.normpath(path)
        originalUrl = uriparse.localPathToURI(path)
    originalProtocol, junk = urllib.splittype( originalUrl )

    if originalProtocol in exceptForProtocols:
        return originalUrl
    else:
        return prefix + urllib.quote( originalUrl )