from xpcom import components, ServerException
import sys
import keyword
from koLanguageServiceBase import *
import os
import StringIO  # cStringIO doesn't work well with unicode strings! 

import autoinsert

class koPythonLexerLanguageService(KoLexerLanguageService):
    def __init__(self):
        KoLexerLanguageService.__init__(self)
        self.setLexer(components.interfaces.ISciMoz.SCLEX_PYTHON)
        if 'None' not in keyword.kwlist:
            self.setKeywords(0, keyword.kwlist + ['None'])
        else:
            self.setKeywords(0, keyword.kwlist)
        self.setProperty('tab.timmy.whinge.level', '1') # XXXX make a user-accesible pref
        # read lexPython.cxx to understand the meaning of the levels.
        self.supportsFolding = 1

# taken from IDLE
# Look at the leading whitespace in s.
# Return pair (# of leading ws characters,
#              effective # of leading blanks after expanding
#              tabs to width tabwidth)

def classifyws(s, tabwidth):
    foundTabs = 0
    raw = effective = 0
    for ch in s:
        if ch == ' ':
            raw = raw + 1
            effective = effective + 1
        elif ch == '\t':
            foundTabs = 1
            raw = raw + 1
            effective = (effective / tabwidth + 1) * tabwidth
        else:
            break
    return raw, effective, foundTabs

import tokenize
_tokenize = tokenize
del tokenize

# taken from IDLE
class IndentSearcher:
    # .run() chews over the Text widget, looking for a block opener
    # and the stmt following it.  Returns a pair,
    #     (line containing block opener, line containing stmt)
    # Either or both may be None.

    def __init__(self, text, tabwidth):
        self.text = StringIO.StringIO(text)
        self.tabwidth = tabwidth
        self.i = self.finished = 0
        self.blkopenline = self.indentedline = None

    def readline(self):
        line = self.text.readline()
        return line

    def tokeneater(self, type, token, start, end, line,
                   INDENT=_tokenize.INDENT,
                   NAME=_tokenize.NAME,
                   OPENERS=('class', 'def', 'for', 'if', 'try', 'while')):
        if self.finished:
            pass
        elif type == NAME and token in OPENERS:
            self.blkopenline = line
        elif type == INDENT and self.blkopenline:
            self.indentedline = line
            self.finished = 1

    def run(self):
        save_tabsize = _tokenize.tabsize
        _tokenize.tabsize = self.tabwidth
        try:
            try:
                _tokenize.tokenize(self.readline, self.tokeneater)
            except _tokenize.TokenError:
                # since we cut off the tokenizer early, we can trigger
                # spurious errors
                pass
        finally:
            _tokenize.tabsize = save_tabsize
        return self.blkopenline, self.indentedline

class koPythonLanguage(KoLanguageBase):
    name = "Python"
    _reg_desc_ = "%s Language" % (name)
    _reg_contractid_ = "@activestate.com/koLanguage?language=%s;1" \
                       % (name)
    _reg_clsid_ = "{D90FF5C7-1FD4-4535-A0D2-47B5BDC3E7FE}"

    namedBlockRE = "^[ \t]*?(def\s+[^\(]+\([^\)]*?\):|class\s+[^:]*?:)"
    namedBlockDescription = 'Python functions and classes'
    defaultExtension = ".py"
    # XXX read url from some config file
    downloadURL = 'http://www.ActiveState.com/Products/ActivePython'
    remoteDebuggerSupport = 1
    commentDelimiterInfo = { "line": [ "#" ]  }
    _comment_styles = [1, 12]
    _string_styles = [3, 4]
    _line_comment_styles = [1, 12]
    _indent_open_chars = ':{[('
    _lineup_open_chars = "([{" 
    _lineup_close_chars = ")]}" 
    _lineup_close_styles = [10]  # may be overriden by the language implementation
    _dedenting_statements = [u'raise', u'return', u'pass', u'yield', u'break', u'continue']
    supportsSmartIndent = "python"
    sample = """import math
class Class:
    \"\"\" this is a docstring \"\"\"
    def __init__(self, arg): # one line comment
        self.x = arg + math.cos(arg + arg)
        s = 'a string'
"""
    
    def get_debugger(self):
        if self._debugger is None:
            self._debugger = KoDebuggerLanguageService(self.name)
        return self._debugger

    def get_lexer(self):
        if self._lexer is None:
            self._lexer = koPythonLexerLanguageService()
        return self._lexer


    def get_linter(self):
        if self._linter is None:
            self._linter = KoLinterLanguageService("@activestate.com/koPythonLinter;1")
        return self._linter

    def get_methodcompletion(self):
        if self._methodcompletion is None:
            self._methodcompletion = components.classes["@activestate.com/koPyCompletionLanguageService;1"].getService(components.interfaces.koICompletionLanguageService)
        return self._methodcompletion

    def get_commenter(self):
        if self._commenter is None:
            self._commenter = KoCommenterLanguageService(
                self.commentDelimiterInfo)
        return self._commenter

    def get_inserter(self):
        if self._inserter is None:
            self._inserter = KoWSInsertLanguageService( \
                autoinsert.PythonAutoInsert(),
                "%s = WebService.ServiceProxy(\"%s\", \"%s\")",
                "WebService", 1)
        return self._inserter
    
    def get_interpreter(self):
        if self._interpreter is None:
            self._interpreter = components.classes["@activestate.com/koPythonInfoEx;1"].getService(components.interfaces.koIAppInfoEx)
        return self._interpreter
    
    def guessIndentation(self, view, tabWidth):
        text = view.text
        # the logic used here (and the code, with minor mods)
        # is taken out of IDLE's AutoIndent extension.
        opener, indented = IndentSearcher(text, tabWidth).run()
        if opener and indented:
            raw, indentsmall, foundTabsInOpener = classifyws(opener, tabWidth)
            raw, indentlarge, foundTabsInIndented = classifyws(indented, tabWidth)
            foundTabs = foundTabsInOpener or foundTabsInIndented
        else:
            indentsmall = indentlarge = 0
            foundTabs = 0
        guess = indentlarge - indentsmall
        return guess, foundTabs

    def findPreviousStatement(self, scimoz, beforeLine):
        """ return the line on which the statement which starts before
        line 'beforeLine' starts, as well as its indentWidth, in a
        two-tuple, or (None, None) if it can't.  The function
        looks back over at most twenty blank or commented lines.
        """
        start = scimoz.getLineEndPosition(beforeLine-1)
        # go back at most 20 lines
        end = scimoz.positionFromLine(max(beforeLine-20, 0))
        p = start+1
        data = scimoz.getStyledText(end, start+1)
        # skip blank lines or lines made up only of comments
        while p > end:
            p -= 1
            ch = data[(p-end)*2]
            if ch in string.whitespace: continue
            style = ord(data[(p-end)*2+1])
            if style in self._comment_styles: continue
            if ch in self._lineup_close_chars and \
               style in self._lineup_close_styles:
                # skip to the match
                braceMatch = scimoz.braceMatch(p)
                if braceMatch == -1:
                    break
                p = braceMatch
                continue # keep looking
            break
        else: # we're running out, we give up
            return None, None
        previousStatementStartOn = self._statementStartingLineFromPos(scimoz, p)
        # then find its indent
        previousIndentWidth = self._getIndentWidthForLine(scimoz,
                                                          previousStatementStartOn)
        return previousStatementStartOn, previousIndentWidth
        
    # this is broken -- it's too general.  Disable for now.
    #def keyPressed(self, ch, scimoz):
    #    # Python-specific: look for :'s that end "secondary" statement
    #    # like 'else' which are
    #    # _not_ indented relative to their parents
    #    # if their parents have a single-line block, that's ok, as in:
    #    #
    #    # if foo: pass
    #    # else:!
    #    #
    #    # but otherwise it's wrong:
    #    # if foo:
    #    #     x = 3
    #    #     else:!
    #    # and the current statement (which may span multiple lines)
    #    # should be dedented to match with the first statement that
    #    # is not aligned with the line above.
    #    if ch == ':':
    #        pos = scimoz.currentPos-1
    #        style = scimoz.getStyleAt(pos)
    #        # we don't want to look at comments
    #        if not style in self._comment_styles:
    #            # first, find the line which starts this statement
    #            thisStatementStartOn = self._statementStartingLineFromPos(scimoz, pos)
    #            # then find its indent
    #            thisIndentWidth = self._getIndentWidthForLine(scimoz, thisStatementStartOn)
    #            # find the line which starts the statement before ours
    #            previousStatementStartOn, previousIndentWidth = \
    #                                      self.findPreviousStatement(scimoz, thisStatementStartOn)
    #            # compare the two
    #            if thisIndentWidth == previousIndentWidth:
    #                # go back in statements until we find a statement
    #                # which is dedented compared to us
    #                lineNo = previousStatementStartOn
    #                thisLine = scimoz.lineFromPosition(pos)
    #                for i in range(20): # go back at most 20 lines
    #                    lineNo, indentWidth = self.findPreviousStatement(scimoz, lineNo)
    #                    if indentWidth is not None and indentWidth < thisIndentWidth:
    #                        # adjust the indentation of all of the lines in
    #                        # "this" statement to match that indentation.
    #                        self.shiftRegionByDelta(scimoz,
    #                                                thisStatementStartOn,
    #                                                thisLine,
    #                                                indentWidth-thisIndentWidth)
    #                        break
    #    # otherwise, defer to the base class.
    #    KoLanguageBase.keyPressed(self, ch, scimoz)
    #
