from xpcom.server.enumerator import SimpleEnumerator
from xpcom import components, ServerException, Exception
from xpcom.nsError import NS_ERROR_NO_INTERFACE

import sys

class koTestRegistry:
    _com_interfaces_ = [components.interfaces.koITestRegistry]
    _reg_clsid_ = "{3CBBB7A7-8F59-46b6-AB1D-0E4A8907DA4D}"
    _reg_contractid_ = "@activestate.com/koTestRegistry;1"
    def __init__(self):
        self._components = []
    def registerComponent(self, suffix):
        comp = components.classes["@activestate.com/koTestable-"+suffix+";1"]\
               .getService(components.interfaces.koITestable)
        self._components.append(comp)
        return comp
    def get_components(self):
        return SimpleEnumerator(self._components)

def listify(enumerator):
    x = []
    while enumerator.hasMoreElements():
        x.append(enumerator.getNext())
    return x

class koTestHarness:
    _com_interfaces_ = [components.interfaces.koITestHarness]
    _reg_clsid_ = "{BE1E8C9E-5B59-4893-B88C-F82D018CB9C3}"
    _reg_contractid_ = "@activestate.com/koTestHarness;1"
    def init(self, stdoutfname, stderrfname):
        self._stdoutfname = stdoutfname
        self._stderrfname = stderrfname
        # do we want to support URIs?
        if stdoutfname:
            self._stdout = components.classes["@activestate.com/koLocalFile;1"].createInstance(components.interfaces.koIFile)
            self._stdout.init(stdoutfname, "w")
        else:
            self._stdout = sys.stdout
        if stderrfname:
            self._stderr = components.classes["@activestate.com/koLocalFile;1"].createInstance(components.interfaces.koIFile)
            self._stderr.init(stderrfname, "w")
        else:
            self._stderr = sys.stderr

    def run(self):
        retval = 0
        failed = 0
        aborted = 0
        numFailures = 0
        numTests = 0
        registry = components.classes["@activestate.com/koTestRegistry;1"].getService(components.interfaces.koITestRegistry);
        cpts = registry.components
        while cpts.hasMoreElements():
            c = cpts.getNext(components.interfaces.koITestable)
            intf = components.interfaces.koIMultiTestable
            try:
                mc = c.QueryInterface(intf)
                numTests += mc.numTests
                testnames, retstrs = mc.multirun()
                for testname, retstr in zip(testnames, retstrs):
                    logLine = c.unitName + '\t' + retstr + '\n'
                    if retstr[:4] == 'PASS':
                        pass
                    elif retstr[:5] == 'ABORT':
                        failed = 1
                        self._stderr.write(logLine)
                        aborted = 1
                        retval = -1
                    elif retstr[:4] == 'FAIL':
                        failed = 1
                        self._stderr.write(logLine)
                        numFailures += 1
                    else:
                        retstr = "BAD TEST RETURN VALUE: " + retstr
                        aborted = 1
                    self._stdout.write(logLine)
                    if aborted:
                        break
            except Exception: # single-test component
                numTests += 1
                retstr = c.run()
                logLine = c.unitName + '\t' + retstr + '\n'
                if retstr[:4] == 'PASS':
                    pass
                elif retstr[:5] == 'ABORT':
                    failed = 1
                    self._stderr.write(logLine)
                    aborted = 1
                    retval = -1
                elif retstr[:4] == 'FAIL':
                    failed = 1
                    self._stderr.write(logLine)
                    numFailures += 1
                else:
                    retstr = "BAD TEST RETURN VALUE: " + retstr
                    aborted = 1
                self._stdout.write(logLine)
                if aborted:
                    break
        self._stdout.close()
        self._stderr.close()
        if aborted:
            return (-1, numTests, numFailures) # on fatal failure
        return failed, numTests, numFailures # on regular run


