import re

class recollector:
    def __init__(self):
        self.res={}
    def add(self, name, reg ):
        re.compile(reg) # check that it is valid

        self.res[name] = reg % self.res
        
collector = recollector()
a = collector.add

a("TextSE" , "[^<]+")
a("UntilHyphen" , "[^-]*-")
a("Until2Hyphens" , "%(UntilHyphen)s(?:[^-]%(UntilHyphen)s)*-")
a("CommentCE" , "%(Until2Hyphens)s>?") 
a("UntilRSBs" , "[^\\]]*](?:[^\\]]+])*]+")
a("CDATA_CE" , "%(UntilRSBs)s(?:[^\\]>]%(UntilRSBs)s)*>" )
a("S" , "[ \\n\\t\\r]+")
a("NameStrt" , "[A-Za-z_:]|[^\\x00-\\x7F]")
a("NameChar" , "[A-Za-z0-9_:.-]|[^\\x00-\\x7F]")
a("Name" , "(?:%(NameStrt)s)(?:%(NameChar)s)*")
a("QuoteSE" , "\"[^\"]*\"|'[^']*'")
a("DT_IdentSE" , "%(S)s%(Name)s(?:%(S)s(?:%(Name)s|%(QuoteSE)s))*" )
a("MarkupDeclCE" , "(?:[^\\]\"'><]+|%(QuoteSE)s)*>" )
a("S1" , "[\\n\\r\\t ]")
a("UntilQMs" , "[^?]*\\?+")
a("PI_Tail" , "\\?>|%(S1)s%(UntilQMs)s(?:[^>?]%(UntilQMs)s)*>" )
a("DT_ItemSE" ,
    "<(?:!(?:--%(Until2Hyphens)s>|[^-]%(MarkupDeclCE)s)|\\?%(Name)s(?:%(PI_Tail)s))|%%%(Name)s;|%(S)s"
)
a("DocTypeCE" ,
"%(DT_IdentSE)s(?:%(S)s)?(?:\\[(?:%(DT_ItemSE)s)*](?:%(S)s)?)?>?" )
a("DeclCE" ,
    "--(?:%(CommentCE)s)?|\\[CDATA\\[(?:%(CDATA_CE)s)?|DOCTYPE(?:%(DocTypeCE)s)?")
a("PI_CE" , "%(Name)s(?:%(PI_Tail)s)?")
a("EndTagCE" , "%(Name)s(?:%(S)s)?>?")
a("AttValSE" , "\"[^<\"]*\"|'[^<']*'")
a("ElemTagCE" ,
    "%(Name)s(?:%(S)s%(Name)s(?:%(S)s)?=(?:%(S)s)?(?:%(AttValSE)s))*(?:%(S)s)?/?>?")

a("MarkupSPE" ,
    "<(?:!(?:%(DeclCE)s)?|\\?(?:%(PI_CE)s)?|/(?:%(EndTagCE)s)?|(?:%(ElemTagCE)s)?)")
a("XML_SPE" , "%(TextSE)s|%(MarkupSPE)s")
a("XML_MARKUP_ONLY_SPE" , "%(MarkupSPE)s")


def lexxml(data, markuponly=0):
    if markuponly:
        reg = "XML_MARKUP_ONLY_SPE"
    else:
        reg = "XML_SPE"
    regex = re.compile(collector.res[reg])
    return regex.findall(data)

def assertlex(data, numtokens, markuponly=0):
    if len(lexxml(data, markuponly))!=numtokens:
        assert len(lexxml(data))==numtokens, "data = '%s', numtokens = '%s'" %(data, numotkens)

def testlexer():
    # this test suite could be larger!
    assertlex("<abc/>", 1)
    assertlex("<abc><def/></abc>", 3)
    assertlex("<abc>Blah</abc>", 3)
    assertlex("<abc>Blah</abc>", 2, markuponly=1)
    assertlex("<?xml version='1.0'?><abc>Blah</abc>", 3, markuponly=1)
    assertlex("<abc>Blah&foo;Blah</abc>", 3)
    assertlex("<abc>Blah&foo;Blah</abc>", 2, markuponly=1)
    assertlex("<abc><abc>", 2)
    assertlex("</abc></abc>", 2)
    assertlex("<abc></def></abc>", 3)

PARENT, TEXT, CHILDREN = 0, 1, 2
def parent(node):
    "parent of this node"
    return node[PARENT]

def text(node):
    return node[TEXT]

def children(node):
    return node[CHILDREN]
    
# The basic data structure used in here is a tuple.
# An API to it is defined above but in the code below I don't
#   use the API 
class fastxmltree:
    def __init__(self, data, prune=1, 
        PARENT=PARENT, TEXT=TEXT, CHILDREN=CHILDREN):
        # the above line is an efficiency hack
        target = dummy = current = (None, "#root", [])

        # we keep a dictionary of tags available for intellisense
        # purposes
        tags = {}
        
        for x in lexxml(data, markuponly=1):
            if x.startswith("<!") or x.startswith("<?"):
                pass
            elif x.startswith("</"):
                recovery = current
                mytagname = parse_etagname(x)
                # walk up the tree looking for a matching start-tag
                # do this to try to recover from "extra" start-
                while current[PARENT] and current[PARENT] is not dummy and tagname(current)!=mytagname:
                    current = current[PARENT]

                # if I never found a matching start-tag, revert back
                # and pretend this end-tag never happened
                if mytagname!=tagname(current):
                    current = recovery
                else:
                    # save memory by deleting nodes we aren't interested in.
                    # if we've seen an end-tag, this isn't a "live" node
                    # anymore, so its children aren't interesting to us.
                    if prune:
                        # clear the children array
                        del current[CHILDREN][:]
                    # try to go back up the stack
                    current = current[PARENT]
                    target = current
            elif x.startswith("<"):
                newnode = (current, x, [])
                current[CHILDREN].append(newnode)
                tags[tagname(newnode)] = None
                if not x.endswith("/>"):
                    target = current = newnode

        # did I find any elements at all?
        if len(children(dummy))>0:
            root = children(dummy)[-1]
        else:
            target = root = None
        self.root = root
        if target and target[TEXT]=="#root":
            self.current = None
        else:
            self.current  = target
        try:
            # sometimes we get an empty tag if the input is not
            # well-formed
            del tags[""]
        except KeyError:
            pass
        self.tags = tags.keys()

def printnode(node, index=1):
    for i in children(node):
        printnode(i, index+1)


def namespace(node):   
    return namespaces(node).get(prefix(node))

namere = re.compile(collector.res["Name"])
def tagname(node):
    if len(node[TEXT])>1:
        return namere.match(node[TEXT][1:]).group()
    else:
        return ""

def parse_etagname(text):
    if len(text)>2:
        return namere.match(text[2:]).group()
    else:
        return ""

attrfinder = re.compile(
    "%(S)s(%(Name)s)(?:%(S)s)?=(?:%(S)s)?(%(AttValSE)s)"
% collector.res)

def attrs(node):
    return attrfinder.findall(text(node))

def prefix(node):
    "return the prefix for an element"
    parts = tagname(node).split(":")
    if len(parts)==1:
        return ""
    else:
        return parts[0]
    
def localname(node):
    "return the localname for an element"
    parts = tagname(node).split(":")
    if len(parts)==1:
        return tagname(node)
    else:
        return parts[1]
    
def prefix_for(node, url):
    "find an XML namespace prefix for a given a node and a namespace URL"
    names = namespaces(node)
    for prefix, namespace in names.items():
       if url==namespace:
           return prefix
    return ""

def namespaces(node):
    "list the namespaces for a node as a dictionary"
    namespaces = {}
    while node is not None:
        for attrname, attrval in attrs(node):
            if attrname.startswith("xmlns"):
                if len(attrname)>5 and attrname[5]==":":
                    prefix = attrname[6:]
                else:
                    prefix = ""
                namespaces.setdefault(prefix, attrval[1:-1]) # strip quotes
        node = parent(node)
    return namespaces

if __name__=="__main__":
    data = """
<xsl:foo xmlns="default ns" xmlns:xsl="xsl namespace" xmlns:blah='blah namespace'>
   <incomplete blah="blah"
   <xsl2:foo xmlns:xsl2="xsl namespace">
      <bogus xmlns:something="something" 
            xmlns:blah="not blah"
>   </random-end-tag>
      </bogus>
      </random-end-tag>
      <bogus>
          <false-start-tag-to-test-recovery>
              <another-false-start-tag>
                  <a-child-element/>
          </random-end-tag>            
      </bogus>
      </random-end-tag>

      <spam xmlns:xsl="another xsl namespace"> <bogus1/> <bogus2> </bogus2> abc 
"""
    testlexer()
    import sys
    import time
    if len(sys.argv)>1:
        data = open(sys.argv[1]).read()
        start = time.time()
        tree = fastxmltree(data)
        ns = namespaces(tree.current)
    else:
        tree = fastxmltree(data)
        root, current = tree.root, tree.current
        assert tagname(current)=="spam"
        assert "xmlns:xsl" in [x for x,y in attrs(current)]
        assert namespaces(current)["xsl"] == "another xsl namespace"
        assert namespaces(current)["xsl2"] == "xsl namespace"
        assert "something" not in namespaces(current).keys()
        assert tagname(children(parent(current))[0]) == "bogus"
        assert tagname(current) == "spam"
        assert namespace(current) == "default ns"
        assert tagname(parent(current)) == "xsl2:foo"
        assert tagname(root) == "xsl:foo"
        assert namespace(root) == "xsl namespace"
        assert namespaces(root)[""] == "default ns"
        assert namespaces(root)["xsl"] == "xsl namespace"

        data = """
        <abc>
            <def>
                <blah """
        tree = fastxmltree(data)
        assert tagname(tree.current) == "blah"

        data = """
        <abc>
            <def>
                <blah a="foo" """
        tree = fastxmltree(data)
        assert tagname(tree.current) == "blah"

        data = """
        <abc>
            <def>
                <blah a="foo"> <"""
        tree = fastxmltree(data)
        assert tagname(tree.current) == ""

        data = """<foo/>"""
        tree = fastxmltree(data)
        assert not tree.current
        
        data = """</unbalanced>
        <a>"""
        tree = fastxmltree(data)
        assert tagname(tree.current)=="a"

        data = """</unbalanced>
        <a>
        <b>"""
        tree = fastxmltree(data)
        assert tagname(tree.current)=="b"

        data = """
        <multi-root></multi-root>
        <multi-root2></multi-root2>
        <b>"""
        tree = fastxmltree(data)
        assert tagname(tree.current)=="b"

        data = """</unbalanced>
        <a></a>
        <c></c>
        <b>"""
        tree = fastxmltree(data)
        assert tagname(tree.current)=="b"

        data = "</unbalanced>"
        tree = fastxmltree(data)
        assert not tree.current

        data = "<abc></abc></abc>"
        tree = fastxmltree(data)
        assert not tree.current

        data = "<abc></abc>"
        tree = fastxmltree(data)
        assert not tree.current
