<?xml version="1.0"?>
<xsl:stylesheet 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:http="http://schemas.xmlsoap.org/wsdl/http/"
  xmlns:html="http://www.w3.org/TR/REC-html40"
  xmlns:soap="http://schemas.xmlsoap.org/wsdl/soap/"
  xmlns:soapenc="http://schemas.xmlsoap.org/soap/encoding/"
  xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/"
  xmlns:s2001="http://www.w3.org/2001/XMLSchema"
  xmlns:s200010="http://www.w3.org/2000/10/XMLSchema"
  xmlns:s1999="http://www.w3.org/1999/XMLSchema"
  xmlns:wsdl="http://schemas.xmlsoap.org/wsdl/"
  xmlns:mime="http://schemas.xmlsoap.org/wsdl/mime/"
  version="1.0">
    
    <!-- global parameters -->
    <xsl:param name='hostname'>localhost</xsl:param>
    <xsl:param name='webservice_header'>WebService::</xsl:param>
    <xsl:param name='expectedService' select='""'/>

    <xsl:output method="text" indent='no' omit-xml-declaration='yes' />
    
    <xsl:strip-space elements='*'/>
    
    <!-- requires that the wsdl was retreived via pass1.xsl -->
    <xsl:template match='wsdl'>
        <xsl:variable name='serviceName'>
            <xsl:call-template name='cleanName'>
                <xsl:with-param name='text' select='wsdl:definitions/wsdl:service/@name'/>
            </xsl:call-template>
        </xsl:variable>
        
        <xsl:text>
use SOAP::Lite;
use strict -refs;
        </xsl:text>
        <xsl:if test='//wsdl:types'>
        <xsl:text disable-output-escaping='yes'>
package </xsl:text><xsl:value-of select='$webservice_header'/><xsl:text disable-output-escaping='yes'>Schema::Array;

# these functions should be in a support library
# simpleType_Array handles building arrays of string, int, float, etc.
# complexType_Array handles building arrays of complex types.

sub simpleType_Array {
    my $name = shift;
    my $arg = shift;
    my $type = shift;
    
    my @ar;
    foreach my $v (@$arg) {
        push @ar, SOAP::Data->type($type)->name("item" => $v);
    }
    
    my $value = SOAP::Data
                    ->type('SOAP-ENC:Array')
                    ->name($name => \@ar);
    return $value;
}

sub complexType_Array {
    my $name = shift;
    my $arg = shift;
    my $type = shift;
    my $namespace = shift;
    
    my $funcname = "ComplexType_$type";
    $funcname =~ s/\.|-/_/;
    
    my @ar;
    foreach my $v (@$arg) {
        #push @ar, SOAP::Data->type($type)->name("item" => $v);
        push @ar, &amp;$funcname('item',$v);
    }
    
    my $value = SOAP::Data
                    ->type('SOAP-ENC:Array')
                    ->attr({'xmlns:t' => $namespace})
                    ->name($name => \@ar);
    return $value;
}

</xsl:text>
        </xsl:if>
        <xsl:for-each select='wsdl:definitions'>
            <!-- figure out the XMLSchema version -->
            <xsl:variable name='SchemaVersion'>
                <xsl:call-template name='getSchemaVersion'>
                    <xsl:with-param name='node' select='.'/>
                </xsl:call-template>
            </xsl:variable>
            
            <!-- parse complex types -->
            <xsl:apply-templates select='wsdl:types'>
                <xsl:with-param name='serviceName' select='$serviceName'/>
            </xsl:apply-templates>
            
            <!-- parse service description -->
            <xsl:apply-templates select='wsdl:service'>
                <xsl:with-param name='serviceName' select='$serviceName'/>
                <xsl:with-param name='SchemaVersion' select='$SchemaVersion'/>
            </xsl:apply-templates>
        </xsl:for-each>
        
    </xsl:template>

<!-- *********************************************************************** -->
<!-- SCHEMA parsing functions -->
<!-- types children: import | schema -->
    <xsl:template match='wsdl:import'>
      <!--<xsl:if test='@location'>
          <xsl:apply-templates select="document(@location)"/>
      </xsl:if>-->
    </xsl:template>

    <xsl:template name='getSchemaVersion'>
        <xsl:param name='node'/>
        <xsl:variable name='version'>
            <xsl:for-each select='$node/namespace::*'>
            <xsl:variable name='value'>
                <xsl:value-of select='.'/>
            </xsl:variable>
            <xsl:choose>
                <xsl:when test='$value = "http://www.w3.org/2001/XMLSchema"'>
                    <xsl:text>2001</xsl:text>
                </xsl:when>
                <xsl:when test='$value = "http://www.w3.org/1999/XMLSchema"'>
                    <xsl:text>1999</xsl:text>
                </xsl:when>
                <!-- soap::lite only does 1999/2000, but that should be ok -->
                <xsl:when test='$value = "http://www.w3.org/2000/10/XMLSchema"'>
                    <xsl:text>2001</xsl:text>
                </xsl:when>
            </xsl:choose>
            </xsl:for-each>
        </xsl:variable>
        <xsl:choose>
            <xsl:when test='$version = ""'>
                <xsl:text>2001</xsl:text>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select='$version'/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    

<xsl:template name='qNameName'>
    <xsl:param name='qname'/>
    <xsl:choose>
        <xsl:when test="contains($qname,':')">
            <xsl:value-of select="substring-after($qname,':')"/>
        </xsl:when>
        <xsl:otherwise>
            <xsl:value-of select="$qname"/>
        </xsl:otherwise>
    </xsl:choose>
</xsl:template>

<xsl:template match="wsdl:types">
    <xsl:param name='serviceName'/>
    <xsl:text>
package </xsl:text><xsl:value-of select='$webservice_header'/><xsl:value-of select='$hostname'/>::<xsl:value-of select='$serviceName'/>::Schema;
    <xsl:apply-templates select="wsdl:import"/>
    <xsl:apply-templates select="wsdl:schema  | s200010:schema | s2001:schema | s1999:schema"/>
</xsl:template>

<!-- schema children: documentation | complexType | element -->
<xsl:template match="wsdl:schema  | s200010:schema | s2001:schema | s1999:schema">
    <xsl:variable name='targetNamespace' select='@targetNamespace'/>
    <xsl:apply-templates select="wsdl:import"/>
    <xsl:apply-templates select="wsdl:complexType | s200010:complexType | s2001:complexType | s1999:complexType">
        <xsl:with-param name="typename"/>
        <xsl:with-param name="type"/>
    </xsl:apply-templates>
</xsl:template>

<!-- complexType children: documentation | complexContent | sequence | all -->
<xsl:template match="wsdl:complexType | s200010:complexType | s2001:complexType | s1999:complexType">
    <xsl:param name="typename"/>
    <xsl:param name="type"/>
    <xsl:variable name="parent" select=".."/>
    <xsl:variable name="parentname" select="name($parent)"/>
    <xsl:choose>
        <xsl:when test="contains($parentname,'element')">
            <xsl:apply-templates select="wsdl:sequence | wsdl:all | wsdl:complexContent | s200010:sequence | s200010:all  | s200010:complexContent | s2001:sequence | s2001:all | s2001:complexContent | s1999:sequence | s1999:all | s1999:complexContent">
                <xsl:with-param name="typename" select="$typename"/>
                <xsl:with-param name="type" select="@name"/>
            </xsl:apply-templates>
        </xsl:when>
        <xsl:otherwise>
        <xsl:variable name='typeFunctionName'>
            <xsl:call-template name='cleanName'>
                <xsl:with-param name='text' select='@name'/>
            </xsl:call-template>
        </xsl:variable>        
<xsl:text>
sub ComplexType_</xsl:text>
<xsl:value-of select="$typeFunctionName"/>
<xsl:text> {
    my ($name, $param) = @_;
    return </xsl:text>
<xsl:apply-templates select="wsdl:sequence | wsdl:all | wsdl:complexContent | s200010:sequence | s200010:all | s200010:complexContent | s2001:sequence | s2001:all | s2001:complexContent | s1999:sequence | s1999:all | s1999:complexContent">
                <xsl:with-param name="typename"/>
                <xsl:with-param name="type" select="@name"/>
</xsl:apply-templates>
<xsl:text>;
}
</xsl:text>
        </xsl:otherwise>
    </xsl:choose>
</xsl:template>

<!--
    given a type like: 'xsd:string[]' this returns 'string'
    TODO: need to handle multi-dimensional arrays [,]
-->
<xsl:template name="arrayType">
    <xsl:param name='rawtype'/>
    <xsl:variable name='typename'>
        <xsl:call-template name='qNameName'>
            <xsl:with-param name='qname' select='$rawtype'/>
        </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="type">
        <xsl:choose>
            <xsl:when test="contains($typename,'[')">
                <xsl:value-of select="substring-before($typename,'[')"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="$typename"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:variable>
    <xsl:value-of select="$type"/>
</xsl:template>

<!--
    given a type like: 's:SOAPStruct' this returns the namespace for 's',
    which should(?) be the targetNamespace of the schema in which it is defined
-->
<xsl:template name='getNamespace'>
    <xsl:param name='type'/>
    <xsl:variable name='typename'>
        <xsl:call-template name='arrayType'>
            <xsl:with-param name='rawtype' select='$type'/>
        </xsl:call-template>
    </xsl:variable>
    <xsl:choose>
        <xsl:when test="contains($type,':')">
            <xsl:variable name='ns' select="//wsdl:complexType[@name=$typename] | //s200010:complexType[@name=$typename] | //s2001:complexType[@name=$typename] | //s1999:complexType[@name=$typename]"/>
            <xsl:for-each select='$ns[1]'>
                <xsl:value-of select='../@targetNamespace'/>
            </xsl:for-each>
        </xsl:when>
    </xsl:choose>
</xsl:template>


<!--
    given a type generates the perl code to create an array of that type
    handles both arrays of simple types and of complex types
-->
<xsl:template name='generateArrayCallingCode'>
    <xsl:param name='rawtype'/>
    <xsl:variable name='type'>
        <xsl:call-template name='arrayType'>
            <xsl:with-param name='rawtype' select='$rawtype'/>
        </xsl:call-template>
    </xsl:variable>
    <xsl:choose>
        <xsl:when test="//wsdl:complexType[@name=$type] | //s200010:complexType[@name=$type] | //s2001:complexType[@name=$type] | //s1999:complexType[@name=$type]">
            <xsl:variable name='typens'>
                <xsl:call-template name='getNamespace'>
                    <xsl:with-param name='type' select='$rawtype'/>
                </xsl:call-template>
            </xsl:variable>
            <xsl:value-of select='$webservice_header'/><xsl:text>Schema::Array::complexType_Array($name, $param, '</xsl:text>
            <xsl:value-of select='$type'/>
            <xsl:text>', '</xsl:text>
            <xsl:value-of select='$typens'/>
            <xsl:text>')</xsl:text>
        </xsl:when>
        <xsl:otherwise>
            <xsl:value-of select='$webservice_header'/><xsl:text>Schema::Array::simpleType_Array($name, $param, '</xsl:text>
            <xsl:value-of select='$type'/>
            <xsl:text>')</xsl:text>
        </xsl:otherwise>
    </xsl:choose>
</xsl:template>

<!-- sequence children: element -->
<xsl:template match="wsdl:sequence | s200010:sequence | s2001:sequence | s1999:sequence">
    <xsl:param name="typename"/>
    <xsl:param name="type"/>
    <xsl:choose>
        <!-- more than one child element indicates a stucture -->
        <xsl:when test="count(child::node()) > 1">
            <xsl:text>SOAP::Data->name($name)->type('</xsl:text>
            <xsl:value-of select="$type"/>
            <xsl:text>')->value(\SOAP::Data->value(
        </xsl:text>

            <xsl:for-each select='child::node()'>
                <xsl:variable name='elName' select='@name'/>
                <xsl:choose>
                    <xsl:when test='contains(name(),"element")'>
                        <xsl:apply-templates select="."/>
                    </xsl:when>
                    <xsl:otherwise>
                        <!-- deals with choice elements -->
                        <xsl:text>$param->{'</xsl:text><xsl:value-of select='child::node()[1]/@name'/><xsl:text>'}</xsl:text>
                    </xsl:otherwise>
                </xsl:choose>
                <xsl:if test='position() &lt; last()'>,
        </xsl:if>
            </xsl:for-each>

            <xsl:text>))</xsl:text>
        </xsl:when>

        <!-- only one child element with maxoccurs of 1 indicates a stucture (?) -->
        <xsl:when test="child::node()/@maxOccurs = 1">
            <xsl:for-each select='child::node()'>
                <xsl:variable name='elName' select='@name'/>                                   
                <xsl:apply-templates select="."/>
                <xsl:if test='position() &lt; last()'>,
                </xsl:if>
            </xsl:for-each>
        </xsl:when>

        <xsl:otherwise>
            <!-- only one child element with no maxoccurs of 1 indicates an array -->
            <xsl:call-template name='generateArrayCallingCode'>
                <xsl:with-param name='rawtype' select='child::node()/@type'/>
            </xsl:call-template>
        </xsl:otherwise>

    </xsl:choose>
</xsl:template>

<!-- all children: element -->
<xsl:template match="wsdl:all | s200010:all | s2001:all | s1999:all">
    <xsl:param name="typename"/>
    <xsl:param name="type"/>
    <xsl:text>SOAP::Data->name(</xsl:text>
    <xsl:choose>
        <xsl:when test="$typename = ''">
            <xsl:text>$name)->type('</xsl:text>
        </xsl:when>
        <xsl:otherwise>
            <xsl:text>'</xsl:text>
            <xsl:value-of select="$typename"/>
            <xsl:text>')-></xsl:text>
        </xsl:otherwise>
    </xsl:choose>
    <xsl:value-of select="$type"/>
    <xsl:text>')->value(\SOAP::Data->value(
        </xsl:text>

        <xsl:for-each select='child::node()'>
        <xsl:variable name='elName' select='@name'/>                                   
        <xsl:apply-templates select="."/>
        <xsl:if test='position() &lt; last()'>,
        </xsl:if>
        </xsl:for-each>
                                   
    <xsl:text>))</xsl:text>
</xsl:template>

<!-- complexContent children: restriction -->
<xsl:template match="wsdl:complexContent  | s200010:complexContent  | s2001:complexContent | s1999:complexContent">
    <xsl:apply-templates select="wsdl:restriction  | s200010:restriction  | s2001:restriction | s1999:restriction"/>
</xsl:template>

<!-- restriction children: attribute -->
<xsl:template match="wsdl:restriction  | s200010:restriction  | s2001:restriction | s1999:restriction">
    <xsl:variable name="basetype">
        <xsl:call-template name='qNameName'>
            <xsl:with-param name='qname' select='@base'/>
        </xsl:call-template>
    </xsl:variable>
    <xsl:apply-templates select="wsdl:attribute  | s200010:attribute  | s2001:attribute | s1999:attribute">
        <xsl:with-param name="basetype" select="$basetype"/>
    </xsl:apply-templates>
</xsl:template>

<xsl:template match="wsdl:attribute  | s200010:attribute  | s2001:attribute | s1999:attribute">
    <xsl:param name="basetype"/>
    <xsl:if test='$basetype = "Array"'>
        <xsl:call-template name='generateArrayCallingCode'>
            <xsl:with-param name='rawtype' select='@wsdl:arrayType'/>
        </xsl:call-template>
    </xsl:if>
</xsl:template>

<!-- element children: complexType -->
<xsl:template match="wsdl:element  | s200010:element | s2001:element | s1999:element">
    <xsl:param name="typename"/>
    <xsl:param name="type"/>
    <xsl:variable name="parent" select=".."/>
    <xsl:variable name="parentname" select="name($parent)"/>
    <xsl:variable name="mytype">
        <xsl:call-template name='qNameName'>
            <xsl:with-param name='qname' select='@type'/>
        </xsl:call-template>
    </xsl:variable>
    <xsl:choose>
        <xsl:when test="contains($parentname,'schema')">
        <xsl:variable name='typeFunctionName'>
            <xsl:call-template name='cleanName'>
                <xsl:with-param name='text' select='@name'/>
            </xsl:call-template>
        </xsl:variable>        
        
        <xsl:text>
sub ComplexType_</xsl:text>
        <xsl:value-of select="$typeFunctionName"/>
        <xsl:text> {
    my ($name, $param) = @_;
    return </xsl:text>
        <xsl:apply-templates select="wsdl:complexType  | s200010:complexType  | s2001:complexType | s1999:complexType">
            <xsl:with-param name="typename"/>
            <xsl:with-param name="type" select="@name"/>
        </xsl:apply-templates>
        <xsl:text>;
}</xsl:text>
        </xsl:when>
        <xsl:otherwise>
            <xsl:choose>
                <xsl:when test="count(child::node()) > 0">
                    <xsl:apply-templates select="child::node()">
                          <xsl:with-param name="typename" select="@name"/>
                          <xsl:with-param name="type"/>
                    </xsl:apply-templates>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:choose>
                        <xsl:when test="//wsdl:complexType[@name=$mytype] | //s200010:complexType[@name=$mytype] | //s2001:complexType[@name=$mytype] | //s1999:complexType[@name=$mytype]">
                            <xsl:variable name='typeFunctionName'>
                                <xsl:call-template name='cleanName'>
                                    <xsl:with-param name='text' select='$mytype'/>
                                </xsl:call-template>
                            </xsl:variable>        
                            <xsl:text>ComplexType_</xsl:text>
                            <xsl:value-of select="$typeFunctionName"/>
                            <xsl:text>('</xsl:text>
                            <xsl:value-of select="@name"/>
                            <xsl:text>', $param->{'</xsl:text>
                            <xsl:value-of select="@name"/>
                            <xsl:text>'})</xsl:text>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:text>SOAP::Data->name('</xsl:text>
                            <xsl:value-of select="@name"/>
                            <xsl:text>')->type('</xsl:text>
                            <xsl:value-of select="@type"/>
                            <xsl:text>')->value($param->{'</xsl:text>
                            <xsl:value-of select="@name"/>
                            <xsl:text>'})</xsl:text>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:otherwise>
    </xsl:choose>
</xsl:template>

<!-- *********************************************************************** -->
<!-- WSDL parsing functions -->
    <xsl:template name='wrapParameter'>
        <xsl:param name='name'/>
        <xsl:param name='type'/>
        <xsl:param name='isDocLiteral'>1</xsl:param>
                                                       
        <xsl:variable name='variableName'>
            <xsl:call-template name='cleanName'>
                <xsl:with-param name='text' select='$name'/>
            </xsl:call-template>
        </xsl:variable>
        <xsl:variable name='typeFunctionName'>
            <xsl:call-template name='cleanName'>
                <xsl:with-param name='text' select='$type'/>
            </xsl:call-template>
        </xsl:variable>

  my $<xsl:value-of select='$variableName'/>= shift;
        <xsl:choose>
            <xsl:when test="//wsdl:complexType[@name=$type] | //s200010:complexType[@name=$type] | //s2001:complexType[@name=$type] | //s1999:complexType[@name=$type]">
  my $wrapped_<xsl:value-of select='$variableName'/> = ComplexType_<xsl:value-of select='$typeFunctionName'/>('<xsl:value-of select='$name'/>', $<xsl:value-of select='$variableName'/>);
            </xsl:when>
            <xsl:otherwise>
  my $wrapped_<xsl:value-of select='$variableName'/> = SOAP::Data->type('<xsl:value-of select='$type'/>')->name('<xsl:value-of select='$name'/>')->value($<xsl:value-of select='$variableName'/>)<xsl:if
test='$isDocLiteral = 0'>->uri($self->{_uri})</xsl:if>;
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
                                                       
    <xsl:template name='generateCode'>
        <xsl:param name='serviceName'/>
        <xsl:param name='soapAction'/>
        <xsl:param name='opName'/>
        <xsl:param name='opNode'/>
        <xsl:param name='portName'/>

        <xsl:variable name='perlNamespace'>
            <xsl:value-of select='$webservice_header'/><xsl:value-of select='$hostname'/>::<xsl:value-of select='$serviceName'/>
        </xsl:variable>

        <xsl:variable name='localFunctionName'>
            <xsl:call-template name='cleanName'>
                <xsl:with-param name='text' select='$opName'/>
            </xsl:call-template>
        </xsl:variable>

        <xsl:variable name='requestType' select='/wsdl/wsdl:definitions/wsdl:binding[substring-after(@type, ":") = $portName]/wsdl:operation[@name=$opName]/soap:operation/@style'/>                   
        <xsl:variable name='isDocLiteral'>
          <xsl:choose>
            <xsl:when test='$requestType="document"'>1</xsl:when>
            <xsl:otherwise>0</xsl:otherwise>
          </xsl:choose>
        </xsl:variable>

        <xsl:text disable-output-escaping='yes'>
sub </xsl:text><xsl:value-of select='$localFunctionName'/><xsl:text disable-output-escaping='yes'> {
  my $self = shift;</xsl:text>
            <xsl:for-each select='wsdl:part'>
                <xsl:variable name="type">
                    <xsl:call-template name='qNameName'>
                        <xsl:with-param name='qname' select='@type | @element'/>
                    </xsl:call-template>
                </xsl:variable>
                <xsl:choose>
                  <xsl:when test='@type'>
                    <xsl:call-template name='wrapParameter'>
                        <xsl:with-param name='name' select='@name'/>
                        <xsl:with-param name='type' select='$type'/>
                        <xsl:with-param name='isDocLiteral' select='$isDocLiteral'/>
                    </xsl:call-template>
                  </xsl:when>
                  <xsl:when test='//wsdl:element[@name=$type] | //s200010:element[@name=$type] | //s2001:element[@name=$type] | //s1999:element[@name=$type]'>
                      <!-- find the complex element, split out it's memebers into parameters -->
                      <xsl:for-each select='//wsdl:element[@name=$type]/child::node()/child::node()/child::node() | //s200010:element[@name=$type]/child::node()/child::node()/child::node() | //s2001:element[@name=$type]/child::node()/child::node()/child::node() | //s1999:element[@name=$type]/child::node()/child::node()/child::node()'>
                        <xsl:variable name="etype">
                            <xsl:call-template name='qNameName'>
                                <xsl:with-param name='qname' select='@type'/>
                            </xsl:call-template>
                        </xsl:variable>
                        <xsl:call-template name='wrapParameter'>
                            <xsl:with-param name='name' select='@name'/>
                            <xsl:with-param name='type' select='$etype'/>
                            <xsl:with-param name='isDocLiteral' select='$isDocLiteral'/>
                        </xsl:call-template>
                      </xsl:for-each>
                  </xsl:when>
                </xsl:choose>
            </xsl:for-each>
      <xsl:if test='//wsdl:operation[@name=$opName]/wsdl:input/soap:header'>
  my $headers = shift;
  my @headers;
      </xsl:if>
      <xsl:for-each select='//wsdl:operation[@name=$opName]/wsdl:input/soap:header'>
          <xsl:variable name='partName' select='@part'/>
          <xsl:variable name='messageName' select='@message'/>
          <xsl:variable name='headerType'>
            <xsl:call-template name='qNameName'>
                <xsl:with-param name='qname' select='//wsdl:part[@name=$partName]/@type'/>
            </xsl:call-template>
          </xsl:variable>
  push @headers, $self->buildHeader('<xsl:value-of select='$partName'/>','<xsl:value-of select='$headerType'/>',$headers->{'<xsl:value-of select='$partName'/>'})
      if exists $headers->{'<xsl:value-of select='$partName'/>'};
      </xsl:for-each>
  my $soapObj = $self->{_soapObj}->on_action(sub { return '"<xsl:value-of select='$soapAction'/>"';});
  my $res = $soapObj->call(<xsl:choose
><xsl:when test="$isDocLiteral = 1"
>SOAP::Data->name('<xsl:value-of select="$opName"
/>') -> attr({xmlns => $self->{_uri}})</xsl:when>
<xsl:otherwise
>'<xsl:value-of select="$opName"/>'</xsl:otherwise
></xsl:choose
> => (<xsl:for-each select='wsdl:part'>
        <xsl:variable name="type">
            <xsl:call-template name='qNameName'>
                <xsl:with-param name='qname' select='@type | @element'/>
            </xsl:call-template>
        </xsl:variable>
        <xsl:choose>
            <xsl:when test='@type'>
                <xsl:variable name='variableName'>
                    <xsl:call-template name='cleanName'>
                        <xsl:with-param name='text' select='@name'/>
                    </xsl:call-template>
                </xsl:variable>
                <xsl:text>$wrapped_</xsl:text><xsl:value-of select='$variableName'/>
                <xsl:if test='position() &lt; last()'>,</xsl:if>
            </xsl:when>
            <xsl:when test='//wsdl:element[@name=$type] | //s200010:element[@name=$type] | //s2001:element[@name=$type] | //s1999:element[@name=$type]'>
                <!-- find the complex element, split out it's memebers into parameters -->
                <xsl:for-each select='//wsdl:element[@name=$type]/child::node()/child::node()/child::node() | //s200010:element[@name=$type]/child::node()/child::node()/child::node() | //s2001:element[@name=$type]/child::node()/child::node()/child::node() | //s1999:element[@name=$type]/child::node()/child::node()/child::node()'>
                    <xsl:variable name='variableName'>
                        <xsl:call-template name='cleanName'>
                            <xsl:with-param name='text' select='@name'/>
                        </xsl:call-template>
                    </xsl:variable>
                    <xsl:text>$wrapped_</xsl:text><xsl:value-of select='$variableName'/>
                    <xsl:if test='position() &lt; last()'>,</xsl:if>
                </xsl:for-each>
            </xsl:when>
        </xsl:choose>
      </xsl:for-each>
        <xsl:if test='//wsdl:operation[@name=$opName]/wsdl:input/soap:header'>
            <xsl:text>, @headers</xsl:text>
        </xsl:if>
<xsl:text disable-output-escaping='yes'>));
  return </xsl:text>
            <xsl:value-of select='$perlNamespace'/>
            <xsl:text disable-output-escaping='yes'>::Result->new($res);
}
</xsl:text>
    </xsl:template>
    
    <xsl:template match='wsdl:definitions/wsdl:portType/wsdl:operation'>
        <xsl:param name='serviceName'/>
        <xsl:param name='soapAction'/>
        <xsl:param name='opName'/>
        
        <xsl:variable name='messageName' select='substring-after(wsdl:input/@message, ":")'/>
        <xsl:variable name='portTypeName' select='../@name'/>
        <xsl:for-each select='../../wsdl:message[@name=$messageName]'>
            <xsl:call-template name='generateCode'>
                <xsl:with-param name='serviceName' select='$serviceName'/>
                <xsl:with-param name='soapAction' select='$soapAction'/>
                <xsl:with-param name='opName' select='$opName'/>
                <xsl:with-param name='portName' select='$portTypeName'/>
            </xsl:call-template>
        </xsl:for-each>
    </xsl:template>
    
    <xsl:template match='wsdl:definitions/wsdl:binding'>
        <xsl:param name='serviceName'/>
        <xsl:param name='endpoint'/>
        <xsl:param name='location'/>
        <xsl:param name='portName'/>
        
        <xsl:if test='soap:binding or name(child::node()) = "soap:binding"'>
            <xsl:for-each select='wsdl:operation/soap:operation'>
                <xsl:variable name='opName' select='../@name'/>
                <xsl:variable name='opNode' select='parent::node()'/>
                <xsl:variable name='bindingType' select='substring-after(../../@type, ":")'/>
                <xsl:variable name='soapAction'>
                    <xsl:choose>
                        <!-- this seems to be handling an MS bug, need to test in other area's also -->
                        <xsl:when test="contains(@soapAction,'#')">
                            <xsl:value-of select="substring-before(@soapAction,'#')"/>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:value-of select="@soapAction"/>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:variable>
                
                <xsl:apply-templates select='//wsdl:portType[@name=$bindingType]/wsdl:operation[@name=$opName]'>
                    <xsl:with-param name='serviceName' select='$serviceName'/>
                    <xsl:with-param name='soapAction' select='$soapAction'/>
                    <xsl:with-param name='opName' select='$opName'/>
                    <xsl:with-param name='opNode' select='$opNode'/>
                </xsl:apply-templates>
            </xsl:for-each>
        </xsl:if>
    </xsl:template>
    
    <!-- Precondition: Current node: /definition/service/port/soap::address -->
    <!-- Precondition: Called only once per run. -->
    <xsl:template name='emitModuleStart'>
        <xsl:param name='endpoint'/>
        <xsl:param name='serviceName'/>
        <xsl:param name='tns'/>
        <xsl:param name='SchemaVersion'/>
        <xsl:param name='portName'/>

        <xsl:variable name='packageNamespace'>
            <xsl:value-of select='$webservice_header'/><xsl:value-of select='$hostname'/>::<xsl:value-of select='$serviceName'/>
        </xsl:variable>
        <xsl:variable name='portNamespace'>
            <xsl:choose>
                <xsl:when test='$portName = ""'>
                    <xsl:value-of select='$packageNamespace'/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select='$packageNamespace'/>::<xsl:value-of select='$portName'/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
package <xsl:value-of select='$portNamespace'/>;
        <xsl:if test='//wsdl:types'>
foreach (keys %<xsl:value-of select='$packageNamespace'/>::Schema::) {
    no strict "refs";
    next unless defined *{"<xsl:value-of select='$packageNamespace'/>::Schema::$_"}{CODE};
    *{$_} = \&amp;{"<xsl:value-of select='$packageNamespace'/>::Schema::$_"};
}
        </xsl:if>
        <xsl:text>
sub new {
  my $class = shift;
  my $data = {};
  my $endpoint = $data->{_endPoint} = '</xsl:text><xsl:value-of select='$endpoint'/><xsl:text>';
  my $uri = $data->{_uri} = '</xsl:text><xsl:value-of select='$tns'/><xsl:text>';
  # we need to get the schema from definitions somehow, and map to soap lite
  $data->{_soapObj} = SOAP::Lite->new (
                        'xmlschema' => '</xsl:text><xsl:value-of select='$SchemaVersion'/><xsl:text>',
                        'proxy' => $endpoint,
                        'uri' => $uri,
                       );
  # this is how we add namespaces in soap lite.
  #$data->{_soapObj}->serializer->namespaces->{'http://soapinterop.org/xsd'} = 'T';
  #$data->{_soapObj}->serializer->namespaces->{'http://soapinterop.org/'} = 'M';
  return bless $data, $class;
}
</xsl:text>
    <xsl:if test='//soap:header'>
<xsl:text>
sub buildHeader {
    my $self = shift;
    my ($name, $type, $info) = @_;
    my $h;
    my $funcname = "ComplexType_$type";
    $funcname =~ s/\.|-/_/;    
    
    if (defined *{"</xsl:text><xsl:value-of select='$packageNamespace'/><xsl:text>::Schema::$funcname"}{CODE}) {
        $h = SOAP::Header->name($name)->value(&amp;$funcname($name,$info->{'value'}));
    } else {
        $h = SOAP::Header->name($name)->type($type)->value($info->{'value'});
    }
    $h->actor($info->{'actor'}) if $info->{'actor'};
    $h->mustUnderstand($info->{'mustUnderstand'}) if $info->{'mustUnderstand'};
    return $h;
}

        </xsl:text>
        </xsl:if>
    </xsl:template>
    
    <xsl:template name='emitModuleEnd'>
        <xsl:param name='serviceName'/>
        <xsl:variable name='perlNamespace'>
            <xsl:value-of select='$webservice_header'/><xsl:value-of select='$hostname'/>::<xsl:value-of select='$serviceName'/>
        </xsl:variable>
package <xsl:value-of select='$perlNamespace'/>::Result;
    
    <xsl:text>
    
sub new {
  my ($class, $res) = @_;
  my $data = {};
  $data->{_res} = $res;
  return bless $data, $class;
}
      
sub fault {
  my ($self) = @_;
  my $res = $self->{_res};
  return $res->fault;
}


sub result {
  my ($self) = @_;
  return $self->{_res}->result;
}

1;
    </xsl:text>
    </xsl:template>
    
    <xsl:template match='wsdl:service'>
      <xsl:param name='serviceName'/>
      <xsl:param name='SchemaVersion'/>
        <xsl:if test='@name = $expectedService or count(../wsdl:service) = 1'>
            <!-- Assume all physical locations in a service are the same,
                     so use the first.
                  -->
            <xsl:variable name='endpoint' select='wsdl:port[1]/soap:address/@location'/>
            
            <xsl:choose>
                <xsl:when test='count(wsdl:port) > 1'>
                    <xsl:for-each select='wsdl:port/soap:address'>
			<xsl:variable name='foundTNS'>
			    <xsl:choose>
			        <xsl:when test='../@targetNamespace'>
			          <xsl:value-of select='../@targetNamespace'/>
			        </xsl:when>
			        <xsl:otherwise>
			          <xsl:value-of select='/wsdl/wsdl:definitions/@targetNamespace'/>
			        </xsl:otherwise>
			    </xsl:choose>
			</xsl:variable>
                        <xsl:call-template name='emitModuleStart'>
                            <xsl:with-param name='serviceName' select='$serviceName'/>
                            <xsl:with-param name='portName' select='../@name'/>
                            <xsl:with-param name='endpoint' select='$endpoint'/>
                            <xsl:with-param name='tns' select='$foundTNS'/>
                            <xsl:with-param name='SchemaVersion' select='$SchemaVersion'/>
                        </xsl:call-template>
                        <xsl:variable name='bindingName' select='substring-after(../@binding, ":")'/>
                        <xsl:apply-templates select='//wsdl:binding[@name=$bindingName]'>
                            <xsl:with-param name='serviceName' select='$serviceName'/>
                        </xsl:apply-templates>
                    </xsl:for-each>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:call-template name='emitModuleStart'>
                        <xsl:with-param name='serviceName' select='$serviceName'/>
                        <xsl:with-param name='endpoint' select='$endpoint'/>
                        <xsl:with-param name='tns' select='../@targetNamespace'/>
                        <xsl:with-param name='SchemaVersion' select='$SchemaVersion'/>
                    </xsl:call-template>
                    <xsl:for-each select='wsdl:port/soap:address'>
                        <xsl:variable name='bindingName' select='substring-after(../@binding, ":")'/>
                        <xsl:apply-templates select='//wsdl:binding[@name=$bindingName]'>
                            <xsl:with-param name='serviceName' select='$serviceName'/>
                        </xsl:apply-templates>
                    </xsl:for-each>
                </xsl:otherwise>
            </xsl:choose>
            
            <xsl:call-template name='emitModuleEnd'>
                <xsl:with-param name='serviceName' select='$serviceName'/>
            </xsl:call-template>
        </xsl:if>
    </xsl:template>
    





    
<xsl:template name='cleanName'>
    <xsl:param name='text'/>
    <xsl:variable name='tmp'>
        <xsl:call-template name='replace-string'>
            <xsl:with-param name='text' select='$text'/>
            <xsl:with-param name='from' select="'.'"/>
            <xsl:with-param name='to' select="'_'"/>
        </xsl:call-template>
    </xsl:variable>
    <xsl:call-template name='replace-string'>
        <xsl:with-param name='text' select='$tmp'/>
        <xsl:with-param name='from' select="'-'"/>
        <xsl:with-param name='to' select="'_'"/>
    </xsl:call-template>
</xsl:template>

<!-- reusable replace-string function -->
<xsl:template name="replace-string">
    <xsl:param name="text"/>
    <xsl:param name="from"/>
    <xsl:param name="to"/>

    <xsl:choose>
        <xsl:when test="contains($text, $from)">

            <xsl:variable name="before" select="substring-before($text, $from)"/>
            <xsl:variable name="after" select="substring-after($text, $from)"/>
            <xsl:variable name="prefix" select="concat($before, $to)"/>

            <xsl:value-of select="$before"/>
            <xsl:value-of select="$to"/>
            <xsl:call-template name="replace-string">
                <xsl:with-param name="text" select="$after"/>
                <xsl:with-param name="from" select="$from"/>
                <xsl:with-param name="to" select="$to"/>
            </xsl:call-template>
        </xsl:when> 
        <xsl:otherwise>
            <xsl:value-of select="$text"/> 
        </xsl:otherwise>
    </xsl:choose> 
</xsl:template>
    
</xsl:stylesheet>
